import unittest
from elifearticle.article import Component
from elifecrossref import component


class TestComponent(unittest.TestCase):
    def test_do_set_component_permissions_none(self):
        comp = Component()
        self.assertIsNone(component.do_set_component_permissions(comp))

    def test_do_set_component_permissions_false(self):
        comp = Component()
        permission = {}
        comp.permissions = [permission]
        self.assertFalse(component.do_set_component_permissions(comp))



import unittest
from xml.etree import ElementTree
from xml.etree.ElementTree import Element
from elifearticle.article import (
    Article,
    Award,
    Citation,
    Dataset,
    FundingAward,
    Preprint,
)
from elifecrossref import related


class TestDoCitationRelatedItem(unittest.TestCase):
    def test_do_citation_related_item(self):
        citation = Citation()
        citation.publication_type = "data"
        citation.doi = "10.7554/eLife.00666"
        do_result = related.do_citation_related_item(citation)
        self.assertTrue(do_result)

    def test_do_citation_related_item_false(self):
        citation = Citation()
        do_result = related.do_citation_related_item(citation)
        self.assertEqual(do_result, False)


class TestDoDatasetRelatedItem(unittest.TestCase):
    def test_do_dataset_related_item(self):
        dataset = Dataset()
        dataset.accession_id = "something"
        do_result = related.do_dataset_related_item(dataset)
        self.assertTrue(do_result)


class TestDoPreprintRelatedItem(unittest.TestCase):
    def test_do_preprint_related_item(self):
        article = Article()
        article.preprint = Preprint(uri="https://example.org")
        do_result = related.do_preprint_related_item(article)
        self.assertTrue(do_result)


class TestDoRelationsProgram(unittest.TestCase):
    def test_do_relations_program_dataset(self):
        article = Article("10.7554/eLife.00666")
        dataset = Dataset()
        dataset.accession_id = "something"
        article.datasets = [dataset]
        do_result = related.do_relations_program(article)
        self.assertTrue(do_result)

    def test_do_relations_program_citation(self):
        article = Article("10.7554/eLife.00666")
        citation = Citation()
        citation.publication_type = "data"
        citation.doi = "10.7554/eLife.00666"
        article.ref_list = [citation]
        do_result = related.do_relations_program(article)
        self.assertTrue(do_result)

    def test_do_relations_program_preprint(self):
        article = Article("10.7554/eLife.00666")
        article.preprint = Preprint(uri="https://example.org")
        do_result = related.do_relations_program(article)
        self.assertTrue(do_result)

    def test_funding_doi(self):
        "test when an article has a funding DOI"
        article = Article("10.7554/eLife.00666")
        award = Award()
        award.award_id = "award_id"
        award.award_id_type = "doi"
        funding_award = FundingAward()
        funding_award.add_award(award)
        article.funding_awards = [funding_award]
        do_result = related.do_relations_program(article)
        self.assertTrue(do_result)

    def test_no_funding_doi(self):
        "test if article award has no funding DOI"
        article = Article("10.7554/eLife.00666")
        award = Award()
        award.award_id = "award_id"
        funding_award = FundingAward()
        funding_award.add_award(award)
        article.funding_awards = [funding_award]
        do_result = related.do_relations_program(article)
        self.assertEqual(do_result, None)


class TestSetRelationsProgram(unittest.TestCase):
    def test_set_relations_program(self):
        parent = Element("root")
        expected_tag = b"<?xml version='1.0' encoding='utf8'?>\n<rel:program />"
        expected_parent = (
            b"<?xml version='1.0' encoding='utf8'?>\n<root><rel:program /></root>"
        )
        tag = related.set_relations_program(parent, None)
        self.assertEqual(ElementTree.tostring(tag, "utf8"), expected_tag)
        self.assertEqual(ElementTree.tostring(parent, "utf8"), expected_parent)


class TestSetRelatedItemDescription(unittest.TestCase):
    def test_set_related_item_description(self):
        description = "a description"
        parent = Element("root")
        expected_parent = (
            b"<?xml version='1.0' encoding='utf8'?>\n<root>"
            b"<rel:description>%s</rel:description></root>" % bytes(description, "utf8")
        )
        related.set_related_item_description(parent, description)
        self.assertEqual(ElementTree.tostring(parent, "utf8"), expected_parent)


class TestSetRelatedItemWorkRelation(unittest.TestCase):
    def test_set_related_item_work_relation_inter(self):
        related_item_type = "inter_work_relation"
        relationship_type = "references"
        identifier_type = "doi"
        related_item_text = "10.5061/dryad.cv323"
        parent = Element("root")
        expected_parent = (
            b"<?xml version='1.0' encoding='utf8'?>\n<root>"
            b'<rel:%s identifier-type="%s" relationship-type="%s">%s</rel:%s></root>'
            % (
                bytes(related_item_type, "utf8"),
                bytes(identifier_type, "utf8"),
                bytes(relationship_type, "utf8"),
                bytes(related_item_text, "utf8"),
                bytes(related_item_type, "utf8"),
            )
        )
        related.set_related_item_work_relation(
            parent,
            related_item_type,
            relationship_type,
            identifier_type,
            related_item_text,
        )
        self.assertEqual(ElementTree.tostring(parent, "utf8"), expected_parent)



import unittest
from xml.etree import ElementTree
from xml.etree.ElementTree import Element
from elifearticle.article import Article
from elifecrossref import peer_review


class TestSetType(unittest.TestCase):
    def test_set_type(self):
        passes = [
            {"article_type": "_not_yet_supported", "expected": b"<root />"},
            {
                "article_type": "article-commentary",
                "expected": b'<root type="editor-report" />',
            },
            {
                "article_type": "reply",
                "expected": b'<root type="author-comment" />',
            },
            {
                "article_type": "referee-report",
                "expected": b'<root type="referee-report" />',
            },
        ]
        for data in passes:
            parent = Element("root")
            article = Article()
            article.article_type = data.get("article_type")
            peer_review.set_type(parent, article)
            parent_string = ElementTree.tostring(parent, "utf-8")
            self.assertEqual(
                parent_string,
                data.get("expected"),
                "failed on article_type %s" % data.get("article_type"),
            )



import unittest

from elifearticle.article import (
    Article,
    Award,
    Component,
    Citation,
    Dataset,
    FundingAward,
    Contributor,
    Affiliation,
    Preprint,
)

from elifecrossref import generate
from elifecrossref.conf import raw_config, parse_raw_config


class TestGenerateComponentList(unittest.TestCase):
    def setUp(self):
        pass

    def test_component_subtitle_no_face_markup(self):
        """build an article object and component, generate Crossref XML"""
        doi = "10.7554/eLife.00666"
        title = "Test article"
        article = Article(doi, title)
        component = Component()
        component.title = "A component"
        component.subtitle = (
            "A <sc>STRANGE</sc> <italic>subtitle</italic>, "
            + "and this tag is <not_allowed>!</not_allowed>"
        )
        expected_subtitle = (
            "A STRANGE subtitle, and this tag is"
            " &lt;not_allowed&gt;!&lt;/not_allowed&gt;"
        )
        article.component_list = [component]
        # generate the crossrefXML
        c_xml = generate.build_crossref_xml([article])
        crossref_xml_string = c_xml.output_xml()
        self.assertIsNotNone(crossref_xml_string)
        # A quick test just look for the expected string to test for tags and escape characters
        self.assertTrue(expected_subtitle in crossref_xml_string)

    def test_component_subtitle_with_face_markup(self):
        """build an article object and component, generate Crossref XML"""
        doi = "10.7554/eLife.00666"
        title = "Test article"
        article = Article(doi, title)
        component = Component()
        component.title = "A component"
        component.subtitle = (
            "A <sc>STRANGE</sc> <italic>subtitle</italic>, "
            + "and this tag is <not_allowed>!</not_allowed>"
        )
        expected_subtitle = (
            "A <sc>STRANGE</sc> <i>subtitle</i>, and this tag "
            + "is &lt;not_allowed&gt;!&lt;/not_allowed&gt;"
        )
        article.component_list = [component]
        # load a config and override the value
        raw_config_object = raw_config("elife")
        face_markup = raw_config_object.get("face_markup")
        raw_config_object["face_markup"] = "true"
        crossref_config = parse_raw_config(raw_config_object)
        # generate the crossrefXML
        c_xml = generate.CrossrefXML([article], crossref_config, None, True)
        crossref_xml_string = c_xml.output_xml()
        self.assertIsNotNone(crossref_xml_string)
        # A quick test just look for the expected string to test for tags and escape characters
        self.assertTrue(expected_subtitle in crossref_xml_string)
        # now set the config back to normal
        raw_config_object["face_markup"] = face_markup


class TestGenerateContributors(unittest.TestCase):
    def setUp(self):
        pass

    def test_generate_no_contributors(self):
        """Test when an article has no contributors"""
        # build an article object and component, generate Crossref XML
        doi = "10.7554/eLife.00666"
        title = "Test article"
        article = Article(doi, title)
        # generate the crossrefXML
        c_xml = generate.build_crossref_xml([article])
        crossref_xml_string = c_xml.output_xml()
        self.assertIsNotNone(crossref_xml_string)
        # A quick test just look for a string value to test
        self.assertTrue("<contributors" not in crossref_xml_string)

    def test_generate_blank_affiliation(self):
        """Test when a contributor has a blank affiliation"""
        doi = "10.7554/eLife.00666"
        title = "Test article"
        article = Article(doi, title)
        author = Contributor("author", "Surname", "Given names")
        aff = Affiliation()
        aff.text = ""
        author.set_affiliation(aff)
        # generate the crossrefXML
        c_xml = generate.build_crossref_xml([article])
        crossref_xml_string = c_xml.output_xml()
        self.assertIsNotNone(crossref_xml_string)
        # A quick test just look for a string value to test
        self.assertTrue("<affiliation>" not in crossref_xml_string)


class TestGenerateCrossrefSchemaVersion(unittest.TestCase):
    def setUp(self):
        pass

    def test_generate_crossref_schema_version_4_3_5(self):
        self.generate_crossref_schema_version(
            "4.3.5", 'xmlns="http://www.crossref.org/schema/4.3.5"'
        )

    def test_generate_crossref_schema_version_4_3_7(self):
        self.generate_crossref_schema_version(
            "4.3.7", 'xmlns="http://www.crossref.org/schema/4.3.7"'
        )

    def test_generate_crossref_schema_version_4_4_0(self):
        self.generate_crossref_schema_version(
            "4.4.0", 'xmlns="http://www.crossref.org/schema/4.4.0"'
        )

    def test_generate_crossref_schema_version_4_4_1(self):
        self.generate_crossref_schema_version(
            "4.4.1", 'xmlns="http://www.crossref.org/schema/4.4.1"'
        )

    def generate_crossref_schema_version(
        self, crossref_schema_version, expected_snippet
    ):
        """Test non-default crossref schema version"""
        # build an article object and component, generate Crossref XML
        doi = "10.7554/eLife.00666"
        title = "Test article"
        article = Article(doi, title)
        # load a config and override the value
        raw_config_object = raw_config("elife")
        original_schema_version = raw_config_object.get("crossref_schema_version")
        raw_config_object["crossref_schema_version"] = crossref_schema_version
        crossref_config = parse_raw_config(raw_config_object)
        # generate the crossrefXML
        c_xml = generate.CrossrefXML([article], crossref_config, None, True)
        crossref_xml_string = c_xml.output_xml()
        self.assertIsNotNone(crossref_xml_string)
        # A quick test just look for a string value to test
        self.assertTrue(expected_snippet in crossref_xml_string)
        # now set the config back to normal
        raw_config_object["crossref_schema_version"] = original_schema_version


class TestGenerateCrossrefCitationId(unittest.TestCase):
    def setUp(self):
        pass

    def test_ref_list_citation_with_no_id(self):
        """for test coverage an article with a ref_list with a citation that has no id attribute"""
        doi = "10.7554/eLife.00666"
        title = "Test article"
        article = Article(doi, title)
        citation = Citation()
        citation.article_title = "An article title"
        article.ref_list = [citation]
        c_xml = generate.build_crossref_xml([article])
        crossref_xml_string = c_xml.output_xml()
        self.assertTrue('<citation key="1">' in crossref_xml_string)


class TestGenerateCrossrefCitationElocationId(unittest.TestCase):
    def test_ref_list_citation_elocation_id(self):
        """for test coverage for schema where elocation_id goes into first_page element"""
        # load a config and override the value
        raw_config_object = raw_config("elife")
        original_schema_version = raw_config_object.get("crossref_schema_version")
        raw_config_object["crossref_schema_version"] = "4.4.0"
        crossref_config = parse_raw_config(raw_config_object)
        doi = "10.7554/eLife.00666"
        title = "Test article"
        article = Article(doi, title)
        citation = Citation()
        citation.elocation_id = "e00003"
        article.ref_list = [citation]
        c_xml = generate.CrossrefXML([article], crossref_config, None, True)
        crossref_xml_string = c_xml.output_xml()
        self.assertTrue("<first_page>e00003</first_page>" in crossref_xml_string)
        # now set the config back to normal
        raw_config_object["crossref_schema_version"] = original_schema_version


class TestGenerateCrossrefDatasets(unittest.TestCase):
    def setUp(self):
        pass

    def test_set_datasets(self):
        """a basic non-XML example for set_datasets"""
        doi = "10.7554/eLife.00666"
        title = "Test article"
        article = Article(doi, title)
        # dataset_1 example with a uri
        dataset_1 = Dataset()
        dataset_1.dataset_type = "datasets"
        dataset_1.uri = (
            "https://github.com/elifesciences/XML-mapping/blob/master/elife-00666.xml"
        )
        dataset_1.title = "Kitchen sink"
        # dataset_2 example with accession_id
        dataset_2 = Dataset()
        dataset_2.dataset_type = "prev_published_datasets"
        dataset_2.accession_id = "EGAS00001000968"
        # dataset_3 example with doi
        dataset_3 = Dataset()
        dataset_3.dataset_type = "prev_published_datasets"
        dataset_3.doi = "10.5061/dryad.cv323"
        # dataset_4 example with no dataset_type will be treated as a generated dataset by default
        dataset_4 = Dataset()
        dataset_4.uri = "http://cghub.ucsc.edu"
        # dataset_5 example with an unsupported dataset_type
        dataset_5 = Dataset()
        dataset_5.dataset_type = "foo"
        dataset_5.uri = "https://elifesciences.org"
        # dataset_6 example with no identifier, not get added to the XML output, for test coverage
        dataset_6 = Dataset()
        dataset_6.dataset_type = "prev_published_datasets"
        # add the datasets to the article object
        article.add_dataset(dataset_1)
        article.add_dataset(dataset_2)
        article.add_dataset(dataset_3)
        article.add_dataset(dataset_4)
        article.add_dataset(dataset_5)
        article.add_dataset(dataset_6)
        # expected values
        expected_xml_snippet_1 = (
            "<rel:program><rel:related_item><rel:description>Kitchen sink</rel:description>"
            + '<rel:inter_work_relation identifier-type="uri" '
            + 'relationship-type="isSupplementedBy">'
            + "https://github.com/elifesciences/XML-mapping/blob/master/elife-00666.xml"
            + "</rel:inter_work_relation></rel:related_item>"
        )
        expected_xml_snippet_2 = (
            '<rel:related_item><rel:inter_work_relation identifier-type="accession" '
            + 'relationship-type="references">EGAS00001000968</rel:inter_work_relation>'
            + "</rel:related_item>"
        )
        expected_xml_snippet_3 = (
            '<rel:related_item><rel:inter_work_relation identifier-type="doi" '
            + 'relationship-type="references">10.5061/dryad.cv323</rel:inter_work_relation>'
            + "</rel:related_item>"
        )
        expected_xml_snippet_4 = (
            '<rel:related_item><rel:inter_work_relation identifier-type="uri" '
            + 'relationship-type="isSupplementedBy">http://cghub.ucsc.edu'
            + "</rel:inter_work_relation></rel:related_item>"
        )
        expected_xml_snippet_5 = (
            '<rel:related_item><rel:inter_work_relation identifier-type="uri" '
            + 'relationship-type="isSupplementedBy">https://elifesciences.org'
            + "</rel:inter_work_relation></rel:related_item>"
        )
        # generate output
        c_xml = generate.build_crossref_xml([article])
        crossref_xml_string = c_xml.output_xml()
        self.assertIsNotNone(crossref_xml_string)
        # Test for expected strings in the XML output
        self.assertTrue(expected_xml_snippet_1 in crossref_xml_string)
        self.assertTrue(expected_xml_snippet_2 in crossref_xml_string)
        self.assertTrue(expected_xml_snippet_3 in crossref_xml_string)
        self.assertTrue(expected_xml_snippet_4 in crossref_xml_string)
        self.assertTrue(expected_xml_snippet_5 in crossref_xml_string)


class TestGenerateCrossrefDataCitation(unittest.TestCase):
    def setUp(self):
        pass

    def test_ref_list_data_citation_with_pmid(self):
        """
        for test coverage an article with a ref_list with a
        data citation that has a pmid attribute
        """
        doi = "10.7554/eLife.00666"
        title = "Test article"
        article = Article(doi, title)
        citation = Citation()
        citation.data_title = "An data title"
        citation.publication_type = "data"
        citation.pmid = "pmid"
        article.ref_list = [citation]
        expected_contains = (
            "<rel:program><rel:related_item><rel:description>An data title</rel:description>"
            + '<rel:inter_work_relation identifier-type="pmid" relationship-type="references">'
            + "pmid</rel:inter_work_relation></rel:related_item></rel:program>"
        )
        # generate
        c_xml = generate.build_crossref_xml([article])
        crossref_xml_string = c_xml.output_xml()
        # test assertion
        self.assertTrue(expected_contains in crossref_xml_string)


class TestGenerateAbstract(unittest.TestCase):
    def setUp(self):
        self.abstract = (
            '<p><bold><italic><underline><sub><sup>An abstract. <ext-link ext-link-type="uri" '
            + 'xlink:href="http://dx.doi.org/10.1601/nm.3602">Desulfocapsa sulfexigens</ext-link>.'
            + "</sup></sub></underline></italic></bold>"
            + ' <xref ref-type="bibr" rid="bib18">Stock and Wise (1990)</xref>.</p>'
        )

    def test_set_abstract(self):
        """test stripping unwanted tags from abstract"""
        doi = "10.7554/eLife.00666"
        title = "Test article"
        article = Article(doi, title)
        article.abstract = self.abstract
        expected_contains = (
            "<jats:abstract><jats:p>An abstract. Desulfocapsa sulfexigens."
            + " Stock and Wise (1990).</jats:p></jats:abstract>"
        )
        # generate
        crossref_object = generate.build_crossref_xml([article])
        crossref_xml_string = crossref_object.output_xml()
        # test assertion
        self.assertTrue(expected_contains in crossref_xml_string)


class TestGenerateTitles(unittest.TestCase):
    def setUp(self):
        self.title = (
            '<bold>Test article</bold> for <ext-link ext-link-type="uri" '
            + 'xlink:href="http://dx.doi.org/10.1601/nm.3602">Desulfocapsa sulfexigens</ext-link>'
        )

    def test_set_titles(self):
        """test stripping unwanted tags from title"""
        doi = "10.7554/eLife.00666"
        article = Article(doi, self.title)
        expected_contains = (
            "<titles><title>Test article for Desulfocapsa sulfexigens</title></titles>"
        )
        # generate
        crossref_object = generate.build_crossref_xml([article])
        crossref_xml_string = crossref_object.output_xml()
        # test assertion
        self.assertTrue(expected_contains in crossref_xml_string)

    def test_set_titles_face_markup_format(self):
        """test the title using face markup set to true"""
        doi = "10.7554/eLife.00666"
        article = Article(doi, self.title)
        expected_contains = (
            "<titles><title><b>Test article</b> for"
            " Desulfocapsa sulfexigens</title></titles>"
        )
        # generate
        raw_config_object = raw_config("elife")
        face_markup = raw_config_object.get("face_markup")
        raw_config_object["face_markup"] = "true"
        crossref_config = parse_raw_config(raw_config_object)
        crossref_object = generate.CrossrefXML([article], crossref_config, None, True)
        crossref_xml_string = crossref_object.output_xml()
        # test assertion
        self.assertTrue(expected_contains in crossref_xml_string)
        # now set the config back to normal
        raw_config_object["face_markup"] = face_markup


class TestGeneratePreprint(unittest.TestCase):
    def test_article_preprint(self):
        """test for an article with preprint data"""
        doi = "10.7554/eLife.00666"
        article = Article(doi, title="Sample article")
        article.preprint = Preprint(uri="https://example.org/")
        expected_contains = (
            "<rel:program>"
            "<rel:related_item>"
            '<rel:intra_work_relation identifier-type="uri"'
            ' relationship-type="hasPreprint">https://example.org/'
            "</rel:intra_work_relation>"
            "</rel:related_item>"
            "</rel:program>"
        )
        # generate
        crossref_object = generate.build_crossref_xml([article])
        crossref_xml_string = crossref_object.output_xml()
        # test assertion
        self.assertTrue(expected_contains in crossref_xml_string)


class TestGenerateFunding(unittest.TestCase):
    def test_set_funding(self):
        "a basic non-XML example for funding data"
        doi = "10.7554/eLife.00666"
        title = "Test article"
        article = Article(doi, title)
        award = Award()
        award.award_id = "10.13039/501100001824"
        award.award_id_type = "doi"
        funding_award = FundingAward()
        funding_award.add_award(award)
        article.funding_awards = [funding_award]
        # expected values
        expected_xml_snippet = (
            "<rel:program>"
            "<rel:related_item>"
            '<rel:inter_work_relation identifier-type="doi" relationship-type="isFinancedBy">'
            "10.13039/501100001824"
            "</rel:inter_work_relation>"
            "</rel:related_item>"
            "</rel:program>"
        )
        # generate output
        c_xml = generate.build_crossref_xml([article])
        crossref_xml_string = c_xml.output_xml()
        self.assertIsNotNone(crossref_xml_string)
        # Test for expected strings in the XML output
        self.assertTrue(expected_xml_snippet in crossref_xml_string)


if __name__ == "__main__":
    unittest.main()



import unittest
from elifecrossref import utils


class TestUtils(unittest.TestCase):
    def test_allowed_tags(self):
        self.assertIsNotNone(utils.allowed_tags(), "allowed_tags not returned")

    def test_clean_string(self):
        self.assertEqual(utils.clean_string(None), None)
        self.assertEqual(utils.clean_string("-normal_"), "-normal_")
        self.assertEqual(utils.clean_string("/abnormal."), "abnormal")



import os
import unittest
from unittest.mock import patch
from collections import OrderedDict
from xml.etree import ElementTree
from xml.etree.ElementTree import Element
import requests
from elifearticle.article import Article, ClinicalTrial
from elifecrossref import clinical_trials
from tests import FIXTURES_PATH, read_file_content


class FakeResponse:
    def __init__(self, status_code=200):
        self.status_code = status_code
        self.content = None


class TestDoClinicalTrials(unittest.TestCase):
    def test_do_clinical_trials(self):
        article = Article("10.7554/eLife.00666")
        article.clinical_trials = [ClinicalTrial()]
        do_result = clinical_trials.do_clinical_trials(article)
        self.assertTrue(do_result)


class TestSetClinicalTrials(unittest.TestCase):
    @patch.object(clinical_trials, "registry_name_to_doi_map")
    def test_set_clinical_trials(self, fake_name_map):
        fake_name_map.return_value = OrderedDict(
            [("ClinicalTrials.gov", "10.18810/clinical-trials-gov")]
        )
        parent = Element("custom_metadata")
        article = Article("10.7554/eLife.00666")
        clinical_trial = ClinicalTrial()
        clinical_trial.source_id = "ClinicalTrials.gov"
        clinical_trial.source_id_type = "registry-name"
        clinical_trial.document_id = "TEST999"
        clinical_trial.content_type = "preResults"
        article.clinical_trials = [clinical_trial]
        expected = (
            "<custom_metadata>"
            "<ct:program>"
            '<ct:clinical-trial-number registry="10.18810/clinical-trials-gov" type="preResults">'
            "TEST999"
            "</ct:clinical-trial-number>"
            "</ct:program>"
            "</custom_metadata>"
        )
        clinical_trials.set_clinical_trials(
            parent,
            article,
            {"clinical_trials_registries": "https://doi.org/10.18810/registries"},
        )
        rough_string = ElementTree.tostring(parent).decode("utf-8")
        self.assertEqual(rough_string, expected)

    @patch.object(clinical_trials, "registry_name_to_doi_map")
    def test_set_clinical_trials_edge_case(self, fake_name_map):
        """test edge case of crossref-doi type and alternate content-type value"""
        fake_name_map.return_value = OrderedDict(
            [("ClinicalTrials.gov", "10.18810/clinical-trials-gov")]
        )
        parent = Element("custom_metadata")
        article = Article("10.7554/eLife.00666")
        clinical_trial = ClinicalTrial()
        clinical_trial.source_id = "10.18810/clinical-trials-gov"
        clinical_trial.source_id_type = "crossref-doi"
        clinical_trial.document_id = "TEST999"
        clinical_trial.content_type = "post-results"
        article.clinical_trials = [clinical_trial]
        expected = (
            "<custom_metadata>"
            "<ct:program>"
            '<ct:clinical-trial-number registry="10.18810/clinical-trials-gov" type="postResults">'
            "TEST999"
            "</ct:clinical-trial-number>"
            "</ct:program>"
            "</custom_metadata>"
        )
        clinical_trials.set_clinical_trials(
            parent,
            article,
            {"clinical_trials_registries": "https://doi.org/10.18810/registries"},
        )
        rough_string = ElementTree.tostring(parent).decode("utf-8")
        self.assertEqual(rough_string, expected)


class TestRegistryNameMap(unittest.TestCase):
    @patch.object(requests, "get")
    def test_registry_name_to_doi_map(self, fake_get):
        registries_response = FakeResponse()
        registries_response.content = "<xml />"
        fake_get.return_value = registries_response
        # expected value
        expected = OrderedDict()
        # issue call for the registries XML
        registry_url = "https://example.org"
        name_map = clinical_trials.registry_name_to_doi_map(registry_url)
        # assertions
        self.assertEqual(name_map, expected)


class TestParseRegistriesXml(unittest.TestCase):
    def test_parse_registries_xml(self):
        expected = OrderedDict(
            [
                ("UTN", "10.18810/utn"),
                ("ClinicalTrials.gov", "10.18810/clinical-trials-gov"),
                ("ISRCTN", "10.18810/isrctn"),
                ("ANZCTR", "10.18810/anzctr"),
                ("DRKS", "10.18810/drks"),
                ("ChiCTR", "10.18810/chictr"),
                ("ReBec", "10.18810/rebec"),
                ("NTR", "10.18810/dutch-trial-register"),
                ("CTRI", "10.18810/clinical-trial-registry-india"),
                ("UMIN", "10.18810/umin-japan"),
                ("PACTR", "10.18810/pactr"),
                ("SLCTR", "10.18810/slctr"),
                ("JMACCT", "10.18810/jma"),
                ("IRCT", "10.18810/irct"),
                ("CRiS", "10.18810/cris"),
                ("RPCEC", "10.18810/rpec"),
                ("EU-CTR", "10.18810/euctr"),
                ("JPRN", "10.18810/jprn"),
                ("TCTR", "10.18810/tctr"),
            ]
        )
        name_map = clinical_trials.parse_registries_xml(
            read_file_content(
                os.path.join(FIXTURES_PATH, "clinical_trial_registries.xml")
            )
        )
        self.assertEqual(name_map, expected)



import re


def allowed_tags():
    """tuple of whitelisted tags"""
    return (
        "<p>",
        "</p>",
        "<italic>",
        "</italic>",
        "<bold>",
        "</bold>",
        "<underline>",
        "</underline>",
        "<sub>",
        "</sub>",
        "<sup>",
        "</sup>",
        "<sc>",
        "</sc>",
        "<inline-formula>",
        "</inline-formula>",
        "<mml:",
        "</mml:",
        "<ext-link",
        "</ext-link>",
        "<xref",
        "</xref>",
        "<sec",
        "</sec>",
        "<title",
        "</title>",
        "<related-object",
        "</related-object>",
    )


def clean_string(string):
    """remove unwanted characters when concatenating the batch_id"""
    if string:
        return re.sub(r"[^a-zA-Z0-9_\-]", "", str(string))
    return None



import time
import os
from xml.etree import ElementTree
from xml.etree.ElementTree import Element, Comment
from xml.dom import minidom

from elifearticle import utils as eautils
from elifearticle import parse

from elifecrossref import body, head, utils

from elifecrossref.conf import raw_config, parse_raw_config


TMP_DIR = "tmp"


class CrossrefXML:
    def __init__(
        self,
        poa_articles,
        crossref_config,
        pub_date=None,
        add_comment=True,
        submission_type="journal",
    ):
        """
        Set the root node
        set default values for dates and batch id
        then build out the XML using the article objects
        """
        # Create the root XML node
        self.root = Element("doi_batch")
        set_root(self.root, crossref_config.get("crossref_schema_version"))

        # Publication date
        if pub_date is None:
            self.pub_date = time.gmtime()
        else:
            self.pub_date = pub_date

        self.batch_id = get_batch_id(
            crossref_config.get("batch_file_prefix"),
            self.pub_date,
            poa_articles,
            submission_type,
        )

        # set comment
        if add_comment:
            self.generated = time.strftime("%Y-%m-%d %H:%M:%S")
            last_commit = eautils.get_last_commit_to_master()
            comment = Comment(
                "generated by "
                + str(crossref_config.get("generator"))
                + " at "
                + self.generated
                + " from version "
                + last_commit
            )
            self.root.append(comment)

        # Build out the Crossref XML
        self.build(poa_articles, crossref_config, submission_type)

    def build(self, poa_articles, crossref_config, submission_type):
        head.set_head(self.root, self.batch_id, self.pub_date, crossref_config)
        body.set_body(
            self.root, poa_articles, crossref_config, self.pub_date, submission_type
        )

    def output_xml(self, pretty=False, indent=""):
        encoding = "utf-8"

        rough_string = ElementTree.tostring(self.root, encoding)
        reparsed = minidom.parseString(rough_string)

        if pretty is True:
            return reparsed.toprettyxml(indent, encoding=encoding).decode(encoding)
        return reparsed.toxml(encoding=encoding).decode(encoding)


def set_root(root, schema_version):
    """Set the root tag namespaces and schema details

    :param root: ElementTree.Element tag
    :param schema_version: version of the Crossref schema as a string, e.g. 4.4.1
    """
    root.set("version", schema_version)
    root.set("xmlns", "http://www.crossref.org/schema/%s" % schema_version)
    root.set("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance")
    root.set("xmlns:fr", "http://www.crossref.org/fundref.xsd")
    root.set("xmlns:ai", "http://www.crossref.org/AccessIndicators.xsd")
    if schema_version != "4.3.5":
        root.set("xmlns:ct", "http://www.crossref.org/clinicaltrials.xsd")
        root.set("xmlns:rel", "http://www.crossref.org/relations.xsd")
    schema_location_name = "http://www.crossref.org/schema/%s" % schema_version
    schema_location_uri = (
        "http://www.crossref.org/schemas/crossref%s.xsd" % schema_version
    )
    root.set(
        "xsi:schemaLocation", "%s %s" % (schema_location_name, schema_location_uri)
    )
    root.set("xmlns:mml", "http://www.w3.org/1998/Math/MathML")
    root.set("xmlns:jats", "http://www.ncbi.nlm.nih.gov/JATS1")


def get_batch_id(batch_file_prefix, pub_date, poa_articles, submission_type):
    """generate a doi_batch_id value for the Crossref deposit"""
    batch_id_parts = []
    # add detail about submission type
    if submission_type != "journal":
        batch_id_parts.append(submission_type)
    # add detail about articles
    if poa_articles:
        # If only one article is supplied, then add the doi to the batch file name
        batch_id_parts.append(str(utils.clean_string(poa_articles[0].manuscript)))
    # add detail about the date
    batch_id_parts.append(time.strftime("%Y%m%d%H%M%S", pub_date))
    # concatenate and return the final batch id
    return str(batch_file_prefix) + "-".join([part for part in batch_id_parts if part])


def build_crossref_xml(
    poa_articles,
    crossref_config=None,
    pub_date=None,
    add_comment=True,
    submission_type="journal",
):
    """
    Given a list of article article objects
    generate crossref XML from them
    """
    if not crossref_config:
        crossref_config = parse_raw_config(raw_config(None))
    return CrossrefXML(
        poa_articles, crossref_config, pub_date, add_comment, submission_type
    )


def crossref_xml(
    poa_articles,
    crossref_config=None,
    pub_date=None,
    add_comment=True,
    submission_type="journal",
    pretty=False,
    indent="",
):
    """build crossref xml and return output as a string"""
    if not crossref_config:
        crossref_config = parse_raw_config(raw_config(None))
    c_xml = build_crossref_xml(
        poa_articles, crossref_config, pub_date, add_comment, submission_type
    )
    return c_xml.output_xml(pretty=pretty, indent=indent)


def crossref_xml_to_disk(
    poa_articles,
    crossref_config=None,
    pub_date=None,
    add_comment=True,
    submission_type="journal",
    pretty=False,
    indent="",
):
    """build crossref xml and write the output to disk"""
    if not crossref_config:
        crossref_config = parse_raw_config(raw_config(None))
    c_xml = build_crossref_xml(
        poa_articles, crossref_config, pub_date, add_comment, submission_type
    )
    xml_string = c_xml.output_xml(pretty=pretty, indent=indent)
    # Write to file
    filename = TMP_DIR + os.sep + c_xml.batch_id + ".xml"
    with open(filename, "wb") as open_file:
        open_file.write(xml_string.encode("utf-8"))


def build_articles_for_crossref(article_xmls, detail="full", build_parts=None):
    """specify some detail and build_parts specific to generating crossref output"""
    build_parts = [
        "abstract",
        "basic",
        "components",
        "contributors",
        "categories",
        "funding",
        "datasets",
        "license",
        "pub_dates",
        "references",
        "related_articles",
        "volume",
        "sub_articles",
        "history",
        "is_poa",
    ]
    return build_articles(article_xmls, detail, build_parts)


def build_articles(article_xmls, detail="full", build_parts=None):
    return parse.build_articles_from_article_xmls(article_xmls, detail, build_parts)



import unittest
import time
import os
import sys
from elifearticle.article import (
    Article,
    ArticleDate,
    Contributor,
    License,
    RelatedObject,
)
from elifecrossref import generate
from tests import (
    DEFAULT_PUB_DATE,
    TEST_BASE_PATH,
    TEST_DATA_PATH,
    helpers,
    read_file_content,
    create_crossref_config,
    replace_namespaces,
)


generate.TMP_DIR = TEST_BASE_PATH + "tmp" + os.sep


def sample_data():
    """some sample data for development until real object definition is final and XML is parsed"""
    reviews = []
    # editor's evaluation
    editor_evaluation = Article()
    editor_evaluation.article_type = "editor-report"
    editor_evaluation.id = "sa0"
    article_author = Contributor("author", "Harrison", "Melissa")
    editor_evaluation.contributors.append(article_author)
    article_collab_author = Contributor(
        "author", None, None, "eLife Editorial Production Group"
    )
    editor_evaluation.contributors.append(article_collab_author)
    editor_evaluation.title = "Editor's evaluation"
    # review date
    review_date = ArticleDate(
        "review_date", time.strptime("2018-01-12 00:00:00", "%Y-%m-%d %H:%M:%S")
    )
    editor_evaluation.add_date(review_date)
    # license
    license_object = License()
    license_object.href = "http://creativecommons.org/licenses/by/4.0/"
    editor_evaluation.license = license_object
    # related article doi
    related_article = Article()
    related_article.doi = "10.7554/eLife.00666"
    related_article.title = "The eLife research article"
    editor_evaluation.related_articles = [related_article]
    # review article doi
    editor_evaluation.doi = "10.7554/eLife.00666.sa0"
    # related material with a uri
    related_object = RelatedObject()
    related_object.xlink_href = (
        "https://sciety.org/articles/activity/10.1101/2020.11.21.391326"
    )
    related_object.link_type = "continued-by"
    editor_evaluation.related_objects = [related_object]
    # a hack to get the resource url right for now
    editor_evaluation.manuscript = "00666#sa0"
    # append it
    reviews.append(editor_evaluation)

    # decision letter
    decision_letter = Article()
    decision_letter.article_type = "editor-report"
    decision_letter.id = "SA1"
    dec_author = Contributor("editor", "Christian", "Rutz")
    decision_letter.contributors.append(dec_author)
    decision_letter.title = "Decision letter"
    # review date
    review_date = ArticleDate(
        "review_date", time.strptime("2018-01-12 00:00:00", "%Y-%m-%d %H:%M:%S")
    )
    decision_letter.add_date(review_date)
    # license
    license_object = License()
    license_object.href = "http://creativecommons.org/licenses/by/4.0/"
    decision_letter.license = license_object
    # related article doi
    related_article = Article()
    related_article.doi = "10.7554/eLife.00666"
    related_article.title = "The eLife research article"
    decision_letter.related_articles = [related_article]
    # review article doi
    decision_letter.doi = "10.7554/eLife.00666.029"
    # a hack to get the resource url right for now
    decision_letter.manuscript = "00666#SA1"
    # append it
    reviews.append(decision_letter)

    # author response
    author_response = Article()
    author_response.article_type = "author-comment"
    author_response.id = "SA2"
    article_author = Contributor("author", "Harrison", "Melissa")
    author_response.contributors.append(article_author)
    article_collab_author = Contributor(
        "author", None, None, "eLife Editorial Production Group"
    )
    author_response.contributors.append(article_collab_author)
    author_response.title = "Author response"
    # review date
    review_date = ArticleDate(
        "review_date", time.strptime("2018-01-12 00:00:00", "%Y-%m-%d %H:%M:%S")
    )
    author_response.add_date(review_date)
    # license
    license_object = License()
    license_object.href = "http://creativecommons.org/licenses/by/4.0/"
    author_response.license = license_object
    # related article doi
    related_article = Article()
    related_article.doi = "10.7554/eLife.00666"
    related_article.title = "The eLife research article"
    author_response.related_articles = [related_article]
    # review article doi
    author_response.doi = "10.7554/eLife.00666.030"
    # a hack to get the resource url right for now
    author_response.manuscript = "00666#SA2"
    # append it
    reviews.append(author_response)

    # anonymous review
    review_article = Article()
    review_article.article_type = "referee-report"
    review_article.id = "SA3"
    article_author = Contributor("author", None, None)
    article_author.anonymous = True
    review_article.contributors.append(article_author)
    review_article.title = "Reviewer #1 (Public Review)"
    # review date
    review_date = ArticleDate(
        "review_date", time.strptime("2018-01-12 00:00:00", "%Y-%m-%d %H:%M:%S")
    )
    review_article.add_date(review_date)
    # license
    license_object = License()
    license_object.href = "http://creativecommons.org/licenses/by/4.0/"
    review_article.license = license_object
    # related article doi
    related_article = Article()
    related_article.doi = "10.7554/eLife.00666"
    related_article.title = "The eLife research article"
    review_article.related_articles = [related_article]
    # review article doi
    review_article.doi = "10.7554/eLife.00666.sa3"
    # a hack to get the resource url right for now
    review_article.manuscript = "00666#SA3"
    # append it
    reviews.append(review_article)

    return reviews


class TestGeneratePeerReview(unittest.TestCase):
    def setUp(self):
        self.passes = []
        self.passes.append(
            (
                "elife-00666.xml",
                "elife-crossref-peer_review-00666-20170717071707.xml",
                "elife",
                DEFAULT_PUB_DATE,
            )
        )

    def test_generate_peer_review(self):
        for (
            article_xml_file,
            crossref_xml_file,
            config_section,
            pub_date,
        ) in self.passes:
            file_path = TEST_DATA_PATH + article_xml_file
            articles = generate.build_articles_for_crossref([file_path])
            # set some review sample data
            articles[0].review_articles = sample_data()
            crossref_config = None
            if config_section:
                crossref_config = create_crossref_config(config_section)
            crossref_xml = generate.crossref_xml(
                articles,
                crossref_config,
                pub_date,
                False,
                "peer_review",
                pretty=True,
                indent="    ",
            )
            model_crossref_xml = read_file_content(TEST_DATA_PATH + crossref_xml_file)
            if sys.version_info < (3, 8):
                # pre Python 3.8 tag attributes are in a different order
                model_crossref_xml = replace_namespaces(model_crossref_xml)
            self.assertEqual(
                crossref_xml,
                model_crossref_xml.decode("utf-8"),
                "Failed parse test on file %s" % article_xml_file,
            )


def preprint_reviews_sample_data():
    "sample data for peer reviews of a preprint"
    reviews = []
    # eLife assessment
    elife_assessment = Article()
    elife_assessment.article_type = "editor-report"
    elife_assessment.id = "sa0"
    article_author = Contributor("author", "Itor", "Ed")
    elife_assessment.contributors.append(article_author)
    elife_assessment.title = "eLife assessment"
    # review date
    review_date = ArticleDate(
        "review_date", time.strptime("2023-05-01 00:00:00", "%Y-%m-%d %H:%M:%S")
    )
    elife_assessment.add_date(review_date)
    # license
    license_object = License()
    license_object.href = "http://creativecommons.org/licenses/by/4.0/"
    elife_assessment.license = license_object
    # related article
    elife_assessment.related_articles = [helpers.build_preprint_article()]
    # review article doi
    elife_assessment.doi = "10.7554/eLife.202200001.2.sa0"
    # append it
    reviews.append(elife_assessment)

    # public review 1
    public_review_1 = Article()
    public_review_1.article_type = "referee-report"
    public_review_1.id = "sa1"
    reviewer = Contributor("author", None, None)
    reviewer.anonymous = True
    public_review_1.contributors.append(reviewer)
    public_review_1.title = "Reviewer #1 (Public Review)"
    # review date
    review_date = ArticleDate(
        "review_date", time.strptime("2023-05-02 00:00:00", "%Y-%m-%d %H:%M:%S")
    )
    public_review_1.add_date(review_date)
    # license
    license_object = License()
    license_object.href = "http://creativecommons.org/licenses/by/4.0/"
    public_review_1.license = license_object
    # related article
    public_review_1.related_articles = [helpers.build_preprint_article()]
    # review article doi
    public_review_1.doi = "10.7554/eLife.202200001.2.sa1"
    # append it
    reviews.append(public_review_1)

    # public review 2
    public_review_2 = Article()
    public_review_2.article_type = "referee-report"
    public_review_2.id = "sa2"
    reviewer = Contributor("author", None, None)
    reviewer.anonymous = True
    public_review_2.contributors.append(reviewer)
    public_review_2.title = "Reviewer #2 (Public Review)"
    # review date
    review_date = ArticleDate(
        "review_date", time.strptime("2023-05-02 00:00:00", "%Y-%m-%d %H:%M:%S")
    )
    public_review_2.add_date(review_date)
    # license
    license_object = License()
    license_object.href = "http://creativecommons.org/licenses/by/4.0/"
    public_review_2.license = license_object
    # related article
    public_review_2.related_articles = [helpers.build_preprint_article()]
    # review article doi
    public_review_2.doi = "10.7554/eLife.202200001.2.sa2"
    # append it
    reviews.append(public_review_2)

    # public review 3
    public_review_3 = Article()
    public_review_3.article_type = "referee-report"
    public_review_3.id = "sa3"
    reviewer = Contributor("author", None, None)
    reviewer.anonymous = True
    public_review_3.contributors.append(reviewer)
    public_review_3.title = "Reviewer #3 (Public Review)"
    # review date
    review_date = ArticleDate(
        "review_date", time.strptime("2023-05-02 00:00:00", "%Y-%m-%d %H:%M:%S")
    )
    public_review_3.add_date(review_date)
    # license
    license_object = License()
    license_object.href = "http://creativecommons.org/licenses/by/4.0/"
    public_review_3.license = license_object
    # related article
    public_review_3.related_articles = [helpers.build_preprint_article()]
    # review article doi
    public_review_3.doi = "10.7554/eLife.202200001.2.sa3"
    # append it
    reviews.append(public_review_3)

    # author response
    author_response = Article()
    author_response.article_type = "author-comment"
    author_response.id = "sa4"
    article_author = Contributor("author", "Or", "Auth")
    author_response.contributors.append(article_author)
    article_collab_author = Contributor("author", None, None, "Research Group")
    author_response.contributors.append(article_collab_author)
    author_response.title = "Author response"
    # review date
    review_date = ArticleDate(
        "review_date", time.strptime("2023-05-03 00:00:00", "%Y-%m-%d %H:%M:%S")
    )
    author_response.add_date(review_date)
    # license
    license_object = License()
    license_object.href = "http://creativecommons.org/licenses/by/4.0/"
    author_response.license = license_object
    # related article
    author_response.related_articles = [helpers.build_preprint_article()]
    # review article doi
    author_response.doi = "10.7554/eLife.202200001.2.sa4"
    # append it
    reviews.append(author_response)

    return reviews


class TestGeneratePreprintPeerReview(unittest.TestCase):
    def setUp(self):
        self.passes = []
        article = helpers.build_preprint_article()
        self.passes.append(
            (
                [article],
                "elife-crossref-preprint-peer_review-202200001-20170717071707.xml",
                "elife_preprint",
                DEFAULT_PUB_DATE,
            )
        )

    def test_generate_peer_review(self):
        for (
            articles,
            crossref_xml_file,
            config_section,
            pub_date,
        ) in self.passes:
            # set some review sample data
            articles[0].review_articles = preprint_reviews_sample_data()
            crossref_config = None
            if config_section:
                crossref_config = create_crossref_config(config_section)
            crossref_xml = generate.crossref_xml(
                articles,
                crossref_config,
                pub_date,
                False,
                "peer_review",
                pretty=True,
                indent="    ",
            )
            model_crossref_xml = read_file_content(TEST_DATA_PATH + crossref_xml_file)
            if sys.version_info < (3, 8):
                # pre Python 3.8 tag attributes are in a different order
                model_crossref_xml = replace_namespaces(model_crossref_xml)
            self.assertEqual(
                crossref_xml,
                model_crossref_xml.decode("utf-8"),
                "Failed parse test on articles %s"
                % ", ".join([article.doi for article in articles]),
            )



import unittest
from xml.etree import ElementTree
from xml.etree.ElementTree import Element
from elifearticle.article import Article, Award, FundingAward
from elifecrossref import funding
from tests import create_crossref_config


class TestDoFunding(unittest.TestCase):
    def test_do_funding(self):
        "test an article with funding"
        # create CrossrefXML object to test
        article = Article("10.7554/eLife.00666", "Test article")
        article.funding_awards = FundingAward()
        # test assertion
        self.assertEqual(funding.do_funding(article), True)

    def test_do_funding_false(self):
        "test and article without funding"
        # create CrossrefXML object to test
        article = Article("10.7554/eLife.00666", "Test article")
        # test assertion
        self.assertEqual(funding.do_funding(article), False)


class TestSetFundref(unittest.TestCase):
    def test_set_fundref(self):
        parent = Element("root")
        article = Article("10.7554/eLife.00666", "Test article")
        award = Award()
        award.award_id = "award_id"
        award.award_id_type = "doi"
        funding_award = FundingAward()
        funding_award.institution_name = "Test Funder"
        funding_award.institution_id = "123456"
        funding_award.id_type = "Fundref"
        funding_award.awards = [award]
        article.funding_awards = [funding_award]
        expected = (
            b"<?xml version='1.0' encoding='utf8'?>\n"
            b"<root>"
            b'<fr:program name="fundref">'
            b'<fr:assertion name="fundgroup">'
            b'<fr:assertion name="funder_name">Test Funder</fr:assertion>'
            b'<fr:assertion name="funder_identifier">123456</fr:assertion>'
            b'<fr:assertion name="award_number">award_id</fr:assertion>'
            b"</fr:assertion>"
            b"</fr:program>"
            b"</root>"
        )
        # invoke
        funding.set_fundref(parent, article)
        parent_string = ElementTree.tostring(parent, "utf8")
        self.assertEqual(parent_string, expected)

    def test_set_fundref_no_funding(self):
        parent = Element("root")
        article = Article("10.7554/eLife.00666", "Test article")
        expected = b"<?xml version='1.0' encoding='utf8'?>\n" b"<root />"
        # invoke
        funding.set_fundref(parent, article)
        parent_string = ElementTree.tostring(parent, "utf8")
        self.assertEqual(parent_string, expected)


class TestSetFinanceRelation(unittest.TestCase):
    def test_set_finance_relation(self):
        parent = Element("fr:program")
        parent.set("xmlns:fr", "http://www.crossref.org/fundref.xsd")
        article = Article("10.7554/eLife.00666", "Test article")
        award = Award()
        award.award_id = "award_id"
        award.award_id_type = "doi"
        funding_award = FundingAward()
        funding_award.institution_name = "Test Funder"
        funding_award.institution_id = "123456"
        funding_award.id_type = "Fundref"
        funding_award.awards = [award]
        article.funding_awards = [funding_award]
        expected = (
            b"<?xml version='1.0' encoding='utf8'?>\n"
            b'<fr:program xmlns:fr="http://www.crossref.org/fundref.xsd">'
            b"<rel:related_item>"
            b'<rel:inter_work_relation identifier-type="doi" relationship-type="isFinancedBy">'
            b"award_id"
            b"</rel:inter_work_relation>"
            b"</rel:related_item>"
            b"</fr:program>"
        )
        # invoke
        funding.set_finance_relation(parent, article)
        parent_string = ElementTree.tostring(parent, "utf8")

        self.assertEqual(parent_string, expected)

    def test_set_finance_relation_no_funding(self):
        parent = Element("root")
        article = Article("10.7554/eLife.00666", "Test article")
        expected = b"<?xml version='1.0' encoding='utf8'?>\n" b"<root />"
        # invoke
        funding.set_finance_relation(parent, article)
        parent_string = ElementTree.tostring(parent, "utf8")
        self.assertEqual(parent_string, expected)



import unittest
from xml.etree import ElementTree
from xml.etree.ElementTree import Element
from elifecrossref import abstract


class TestConvertSecTags(unittest.TestCase):
    def test_convert_sec_tags(self):
        string = (
            '<sec id="s1"><title content-type="sub">Section title:</title>'
            "<p>Paragraph.</p></sec>"
        )
        expected = '<jats:p content-type="sub">Section title:</jats:p><p>Paragraph.</p>'
        self.assertEqual(abstract.convert_sec_tags(string), expected)


class TestGetJatsAbstract(unittest.TestCase):
    def test_get_jats_abstract(self):
        string = (
            "<abstract>"
            '<sec id="s1"><title content-type="sub">Section title:</title>'
            "<p>Paragraph.</p></sec>"
            "</abstract>"
        )
        expected = (
            '<jats:sec id="s1"><jats:title content-type="sub">Section title:</jats:title>'
            "<jats:p>Paragraph.</jats:p></jats:sec>"
        )
        self.assertEqual(abstract.get_jats_abstract(string), expected)

    def test_get_jats_abstract_namespace(self):
        string = (
            "<abstract>"
            "<p>Paragraph</p>"
            "<p><bold>DOI:</bold>"
            '<ext-link ext-link-type="doi" xlink:href="10.7554/eLife.15272.001">'
            "https://doi.org/10.7554/eLife.15272.001</ext-link></p>"
            "</abstract>"
        )
        expected = (
            "<jats:p>Paragraph</jats:p><jats:p><jats:bold>DOI:</jats:bold>"
            '<jats:ext-link ext-link-type="doi" xlink:href="10.7554/eLife.15272.001">'
            "https://doi.org/10.7554/eLife.15272.001</jats:ext-link></jats:p>"
        )
        self.assertEqual(abstract.get_jats_abstract(string), expected)


class TestSetAbstractTag(unittest.TestCase):
    def test_set_abstract_tag(self):
        parent = Element("root")
        string = (
            "<abstract>"
            '<sec id="s1"><title content-type="sub">Section title:</title>'
            "<p>Paragraph.</p></sec>"
            "</abstract>"
        )
        jats_abstract = False
        expected = (
            b"<?xml version='1.0' encoding='utf8'?>\n"
            b"<root>"
            b"<jats:abstract>"
            b'<jats:p content-type="sub">Section title:</jats:p>'
            b"<jats:p>Paragraph.</jats:p>"
            b"</jats:abstract>"
            b"</root>"
        )
        abstract.set_abstract_tag(parent, string, jats_abstract=jats_abstract)
        parent_string = ElementTree.tostring(parent, "utf8")
        self.assertEqual(parent_string, expected)

    def test_set_abstract_tag_jats(self):
        parent = Element("root")
        string = (
            "<abstract>"
            '<sec id="s1"><title content-type="sub">Section title:</title>'
            "<p>Paragraph.</p></sec>"
            "</abstract>"
        )
        jats_abstract = True
        expected = (
            b"<?xml version='1.0' encoding='utf8'?>\n"
            b"<root>"
            b"<jats:abstract>"
            b'<jats:sec id="s1">'
            b'<jats:title content-type="sub">Section title:</jats:title>'
            b"<jats:p>Paragraph.</jats:p>"
            b"</jats:sec>"
            b"</jats:abstract>"
            b"</root>"
        )
        abstract.set_abstract_tag(parent, string, jats_abstract=jats_abstract)
        parent_string = ElementTree.tostring(parent, "utf8")
        self.assertEqual(parent_string, expected)

    def test_set_abstract_tag_mathml(self):
        parent = Element("root")
        string = (
            "<abstract>"
            "<p>This is the abstract."
            "<inline-formula><mml:math><mml:mn>0</mml:mn></mml:math></inline-formula>"
            "</p></abstract>"
        )
        jats_abstract = False
        expected = (
            b"<?xml version='1.0' encoding='utf8'?>\n"
            b"<root>"
            b'<jats:abstract xmlns:mml="http://www.w3.org/1998/Math/MathML">'
            b'<jats:p xmlns:mml="http://www.w3.org/1998/Math/MathML">'
            b"This is the abstract."
            b"<mml:math><mml:mn>0</mml:mn></mml:math>"
            b"</jats:p>"
            b"</jats:abstract>"
            b"</root>"
        )
        abstract.set_abstract_tag(parent, string, jats_abstract=jats_abstract)
        parent_string = ElementTree.tostring(parent, "utf8")
        self.assertEqual(parent_string, expected)

    def test_set_abstract_tag_nested_ext_link(self):
        parent = Element("root")
        string = (
            "<abstract>"
            "<p>Paragraph</p>"
            "<p><bold>"
            '<ext-link ext-link-type="doi" xlink:href="10.7554/eLife.15272.001">'
            "https://doi.org/10.7554/eLife.15272.001</ext-link></bold></p>"
            "</abstract>"
        )
        jats_abstract = True
        expected = (
            b"<?xml version='1.0' encoding='utf8'?>\n"
            b"<root>"
            b'<jats:abstract xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<jats:p>Paragraph</jats:p>"
            b'<jats:p xmlns:xlink="http://www.w3.org/1999/xlink"><jats:bold>'
            b'<jats:ext-link ext-link-type="doi" xlink:href="10.7554/eLife.15272.001">'
            b"https://doi.org/10.7554/eLife.15272.001</jats:ext-link>"
            b"</jats:bold></jats:p>"
            b"</jats:abstract>"
            b"</root>"
        )
        abstract.set_abstract_tag(parent, string, jats_abstract=jats_abstract)
        parent_string = ElementTree.tostring(parent, "utf8")
        self.assertEqual(parent_string, expected)



import unittest
from xml.etree import ElementTree
from xml.etree.ElementTree import Element
from elifearticle.article import Article, Award, FundingAward, License
from elifecrossref import crossmark
from tests import create_crossref_config


class TestDoCrossmark(unittest.TestCase):
    def setUp(self):
        self.crossref_config = create_crossref_config()

    def test_do_crossref(self):
        article = Article("10.7554/eLife.00666")
        do_result = crossmark.do_crossmark(article, self.crossref_config)
        self.assertTrue(do_result)

    def test_do_crossref_false(self):
        article = Article("10.7554/eLife.00666")
        self.crossref_config["crossmark"] = False
        do_result = crossmark.do_crossmark(article, self.crossref_config)
        self.assertFalse(do_result)

    def test_do_crossref_no_doi(self):
        article = Article()
        do_result = crossmark.do_crossmark(article, self.crossref_config)
        self.assertFalse(do_result)


class TestSetCrossmark(unittest.TestCase):
    def setUp(self):
        self.crossref_config = create_crossref_config()
        self.article = Article("10.7554/eLife.00666")
        self.parent = Element("journal_article")

    def test_set_crossmark(self):
        expected = (
            "<journal_article>"
            "<crossmark>"
            "<crossmark_policy>10.7554/eLife.00666</crossmark_policy>"
            "<crossmark_domains>"
            "<crossmark_domain>"
            "<domain>elifesciences.org</domain>"
            "</crossmark_domain>"
            "</crossmark_domains>"
            "<crossmark_domain_exclusive>false</crossmark_domain_exclusive>"
            "</crossmark>"
            "</journal_article>"
        )
        crossmark.set_crossmark(self.parent, self.article, self.crossref_config)
        rough_string = ElementTree.tostring(self.parent).decode("utf-8")
        self.assertEqual(rough_string, expected)

    def test_set_crossmark_crossmark_policy(self):
        """test if an explicity crossmark_policy DOI is specified in config"""
        self.crossref_config["crossmark_policy"] = "10.7554/example"
        expected = (
            "<journal_article>"
            "<crossmark>"
            "<crossmark_policy>10.7554/example</crossmark_policy>"
            "<crossmark_domains>"
            "<crossmark_domain>"
            "<domain>elifesciences.org</domain>"
            "</crossmark_domain>"
            "</crossmark_domains>"
            "<crossmark_domain_exclusive>false</crossmark_domain_exclusive>"
            "</crossmark>"
            "</journal_article>"
        )
        crossmark.set_crossmark(self.parent, self.article, self.crossref_config)
        rough_string = ElementTree.tostring(self.parent).decode("utf-8")
        self.assertEqual(rough_string, expected)

    def test_set_crossmark_domain_filter(self):
        """test config with a domain with a filter value"""
        self.crossref_config["crossmark_domains"][0]["filter"] = "foo"
        expected = (
            "<journal_article>"
            "<crossmark>"
            "<crossmark_policy>10.7554/eLife.00666</crossmark_policy>"
            "<crossmark_domains>"
            "<crossmark_domain>"
            "<domain>elifesciences.org</domain>"
            "<filter>foo</filter>"
            "</crossmark_domain>"
            "</crossmark_domains>"
            "<crossmark_domain_exclusive>false</crossmark_domain_exclusive>"
            "</crossmark>"
            "</journal_article>"
        )
        crossmark.set_crossmark(self.parent, self.article, self.crossref_config)
        rough_string = ElementTree.tostring(self.parent).decode("utf-8")
        self.assertEqual(rough_string, expected)


class TestDoCustomMetadata(unittest.TestCase):
    def setUp(self):
        self.crossref_config = create_crossref_config()
        self.article = Article()

    def test_do_custom_metadata(self):
        self.assertFalse(
            crossmark.do_custom_metadata(self.article, self.crossref_config)
        )

    def test_do_custom_metadata_has_funding(self):
        self.article.funding_awards = ["Mock funding award"]
        self.assertTrue(
            crossmark.do_custom_metadata(self.article, self.crossref_config)
        )

    def test_do_custom_metadata_has_access_indicators(self):
        article_license = License()
        article_license.href = "https://example.org"
        self.article.license = article_license
        self.assertTrue(
            crossmark.do_custom_metadata(self.article, self.crossref_config)
        )


class TestSetCustomMetadata(unittest.TestCase):
    def setUp(self):
        self.crossref_config = create_crossref_config()
        self.article = Article()
        article_license = License()
        article_license.href = "https://example.org"
        self.article.license = article_license
        self.parent = Element("crossmark")

    def test_set_custom_metadata_license_only(self):
        expected = (
            "<crossmark>"
            "<custom_metadata>"
            '<ai:program name="AccessIndicators">'
            '<ai:license_ref applies_to="vor">https://example.org</ai:license_ref>'
            '<ai:license_ref applies_to="am">https://example.org</ai:license_ref>'
            '<ai:license_ref applies_to="tdm">https://example.org</ai:license_ref>'
            "</ai:program>"
            "</custom_metadata>"
            "</crossmark>"
        )
        crossmark.set_custom_metadata(self.parent, self.article, self.crossref_config)
        rough_string = ElementTree.tostring(self.parent).decode("utf-8")
        self.assertEqual(rough_string, expected)

    def test_set_custom_metadata_license_funding(self):
        funding_award = FundingAward()
        funding_award.award_group_id = "group_id"
        award_object = Award()
        award_object.award_id = "award_id"
        funding_award.awards = [award_object]
        funding_award.institution_name = "Institution"
        funding_award.institution_id = "institution_id"
        funding_award.principal_award_recipients = ["Recipient One", "Recipient Two"]

        self.article.funding_awards = [funding_award]
        expected = (
            "<crossmark>"
            "<custom_metadata>"
            '<fr:program name="fundref">'
            '<fr:assertion name="fundgroup">'
            '<fr:assertion name="funder_name">Institution</fr:assertion>'
            '<fr:assertion name="funder_identifier">institution_id</fr:assertion>'
            '<fr:assertion name="award_number">award_id</fr:assertion>'
            "</fr:assertion>"
            "</fr:program>"
            '<ai:program name="AccessIndicators">'
            '<ai:license_ref applies_to="vor">https://example.org</ai:license_ref>'
            '<ai:license_ref applies_to="am">https://example.org</ai:license_ref>'
            '<ai:license_ref applies_to="tdm">https://example.org</ai:license_ref>'
            "</ai:program>"
            "</custom_metadata>"
            "</crossmark>"
        )
        crossmark.set_custom_metadata(self.parent, self.article, self.crossref_config)
        rough_string = ElementTree.tostring(self.parent).decode("utf-8")
        self.assertEqual(rough_string, expected)



from xml.etree.ElementTree import SubElement
from elifecrossref import related

# order matters when choosing the identifier
IDENTIFIER_NAMES = ["doi", "accession_id", "uri"]

IDENTIFIER_MAP = {"doi": "doi", "accession_id": "accession", "uri": "uri"}


def set_datasets(relations_program_tag, poa_article):
    """
    Add related_item tags for each dataset
    """
    for dataset_object in poa_article.datasets:
        # Check for at least one identifier before adding the related_item
        if not related.do_dataset_related_item(dataset_object):
            continue
        set_dataset_related_item(relations_program_tag, dataset_object)


def set_dataset_related_item(parent, dataset_object):
    # add related_item tag
    related_item_tag = SubElement(parent, "rel:related_item")
    related_item_type = "inter_work_relation"
    description = None
    relationship_type = dataset_relationship_type(dataset_object)
    # set the description
    if dataset_object.title:
        description = dataset_object.title
    if description:
        related.set_related_item_description(related_item_tag, description)
    # Now add one inter_work_relation tag in order of priority
    identifier_attr_name = choose_dataset_identifier(dataset_object)
    if identifier_attr_name:
        identifier_type = IDENTIFIER_MAP.get(identifier_attr_name)
        related_item_text = getattr(dataset_object, identifier_attr_name)
        related.set_related_item_work_relation(
            related_item_tag,
            related_item_type,
            relationship_type,
            identifier_type,
            related_item_text,
        )


def choose_dataset_identifier(d_obj):
    """return the name of the first non blank attribute"""
    for attr_name in IDENTIFIER_NAMES:
        if hasattr(d_obj, attr_name) and getattr(d_obj, attr_name):
            return attr_name
    return None


def dataset_relationship_type(dataset_object):
    """relationship_type for the related_item depending on the dataset_type"""
    if dataset_object.dataset_type:
        if dataset_object.dataset_type == "prev_published_datasets":
            return "references"
        if dataset_object.dataset_type == "datasets":
            return "isSupplementedBy"
    # default if not specified
    return "isSupplementedBy"



from xml.etree.ElementTree import SubElement
from elifecrossref import (
    abstract,
    access_indicators,
    collection,
    contributor,
    dates,
    doi,
    related,
    title,
)


def set_posted_content(parent, poa_article, crossref_config):
    "build a Crossref deposit for a posted_content tag"
    posted_content_tag = SubElement(parent, "posted_content")
    posted_content_tag.set("type", "preprint")

    # group_title
    if hasattr(poa_article, "group_title") and poa_article.group_title:
        group_title_tag = SubElement(posted_content_tag, "group_title")
        group_title_tag.text = poa_article.group_title

    # contributors
    contrib_types = ["author"]
    contributor.set_article_contributors(posted_content_tag, poa_article, contrib_types)

    # title is required
    title.set_titles(posted_content_tag, poa_article.title, crossref_config)

    # date is required
    dates.set_posted_date(posted_content_tag, poa_article.get_date("posted_date"))

    # item_number
    if poa_article.elocation_id:
        item_number_tag = SubElement(posted_content_tag, "item_number")
        item_number_tag.set("item_number_type", "article_number")
        item_number_tag.text = poa_article.elocation_id

    # abstract
    abstract.set_abstract(posted_content_tag, poa_article, crossref_config)

    # ai:program license
    if poa_article.license and poa_article.license.href:
        ai_program_tag = access_indicators.set_ai_program(posted_content_tag)
        access_indicators.set_ai_license_ref(ai_program_tag, poa_article.license.href)

    # rel:program related_item tags for preprint versions
    if poa_article.publication_history:
        program_tag = related.set_relations_program(posted_content_tag, None)
        for event in poa_article.publication_history:
            related_item_tag = SubElement(program_tag, "rel:related_item")
            related_item_type = "intra_work_relation"
            relationship_type = "isVersionOf"
            if event.doi:
                identifier_type = "doi"
                related_item_text = event.doi
            elif event.uri:
                identifier_type = "uri"
                related_item_text = event.uri
            related.set_related_item_work_relation(
                related_item_tag,
                related_item_type,
                relationship_type,
                identifier_type,
                related_item_text,
            )

    doi_data_tag = doi.set_doi_data(
        posted_content_tag,
        poa_article,
        poa_article,
        crossref_config,
        pattern_type="doi_pattern",
    )

    collection.set_collection(
        doi_data_tag, poa_article, "crawler-based", crossref_config
    )



from xml.etree.ElementTree import SubElement
from elifecrossref import related


def do_funding(poa_article):
    return bool(poa_article.funding_awards)


def set_fundref(parent, poa_article):
    """
    Set the fundref data from the article funding_awards list
    """
    if do_funding(poa_article):
        fr_program_tag = SubElement(parent, "fr:program")
        fr_program_tag.set("name", "fundref")
    for award in poa_article.funding_awards:
        set_funding_award(fr_program_tag, award)


def set_funding_award(parent, award):
    fr_fundgroup_tag = SubElement(parent, "fr:assertion")
    fr_fundgroup_tag.set("name", "fundgroup")
    fr_funder_name_tag = set_funder_name(fr_fundgroup_tag, award)
    if fr_funder_name_tag is not None:
        set_funder_identifier(fr_fundgroup_tag, award)
    set_award_number(fr_fundgroup_tag, award)


def set_funder_name(parent, award):
    fr_funder_name_tag = None
    if award.get_funder_name():
        fr_funder_name_tag = SubElement(parent, "fr:assertion")
        fr_funder_name_tag.set("name", "funder_name")
        fr_funder_name_tag.text = award.get_funder_name()
    return fr_funder_name_tag


def set_funder_identifier(parent, award):
    if award.get_funder_name() and award.institution_id:
        fr_funder_identifier_tag = SubElement(parent, "fr:assertion")
        fr_funder_identifier_tag.set("name", "funder_identifier")
        fr_funder_identifier_tag.text = award.institution_id


def set_award_number(parent, award):
    for award_object in award.awards:
        fr_award_number_tag = SubElement(parent, "fr:assertion")
        fr_award_number_tag.set("name", "award_number")
        fr_award_number_tag.text = award_object.award_id


def set_finance_relation(relations_program_tag, poa_article):
    "create related_item tags for awards with a funding DOI"
    for award in poa_article.funding_awards:
        for award_object in [
            award_object
            for award_object in award.awards
            if award_object.award_id_type == "doi"
        ]:
            set_finance_related_item(
                relations_program_tag,
                "isFinancedBy",
                award_object.award_id_type,
                award_object.award_id,
            )


def set_finance_related_item(
    parent, relationship_type, identifier_type, related_item_text
):
    "add a related_item tag"
    related_item_tag = SubElement(parent, "rel:related_item")
    related_item_type = "inter_work_relation"
    related.set_related_item_work_relation(
        related_item_tag,
        related_item_type,
        relationship_type,
        identifier_type,
        related_item_text,
    )



import unittest
from xml.etree import ElementTree
from xml.etree.ElementTree import Element
from elifearticle.article import Article, License, Uri
from elifecrossref import collection
from tests import create_crossref_config


class TestSetCollection(unittest.TestCase):
    def test_text_mining_vor(self):
        parent = Element("root")
        crossref_config = create_crossref_config("elife")
        article = Article("10.7554/eLife.00666", "Test article")
        article.manuscript = "00666"
        article.version = "1"
        # self_uri
        uri = Uri()
        uri.xlink_href = "elife-06666.pdf"
        uri.content_type = "pdf"
        article.self_uri_list = [uri]
        # create a license with a href value
        license_object = License()
        license_object.href = "http://example.org/license.txt"
        article.license = license_object
        expected = (
            b"<?xml version='1.0' encoding='utf8'?>\n"
            b"<root>"
            b'<collection property="text-mining">'
            b"<item>"
            b'<resource mime_type="application/pdf">'
            b"https://cdn.elifesciences.org/articles/00666/elife-00666-v1.pdf"
            b"</resource>"
            b"</item>"
            b"<item>"
            b'<resource mime_type="application/xml">'
            b"https://cdn.elifesciences.org/articles/00666/elife-00666-v1.xml"
            b"</resource>"
            b"</item>"
            b"</collection>"
            b"</root>"
        )
        collection.set_collection(parent, article, "text-mining", crossref_config)
        parent_string = ElementTree.tostring(parent, "utf8")
        self.assertEqual(parent_string, expected)

    def test_crawler_based_preprint(self):
        parent = Element("root")
        crossref_config = create_crossref_config("elife_preprint")
        article = Article("10.7554/eLife.00666", "Test article")
        article.manuscript = "00666"
        article.article_type = "preprint"
        expected = (
            b"<?xml version='1.0' encoding='utf8'?>\n"
            b"<root>"
            b'<collection property="crawler-based">'
            b'<item crawler="iParadigms">'
            b"<resource>https://elifesciences.org/reviewed-preprints/00666/pdf</resource>"
            b"</item>"
            b"</collection>"
            b"</root>"
        )
        collection.set_collection(parent, article, "crawler-based", crossref_config)
        parent_string = ElementTree.tostring(parent, "utf8")
        self.assertEqual(parent_string, expected)

    def test_crawler_based_poa(self):
        parent = Element("root")
        crossref_config = create_crossref_config("elife")
        article = Article("10.7554/eLife.00666", "Test article")
        article.manuscript = "00666"
        article.is_poa = True
        expected = (
            b"<?xml version='1.0' encoding='utf8'?>\n"
            b"<root>"
            b'<collection property="crawler-based">'
            b'<item crawler="iParadigms">'
            b"<resource>https://cdn.elifesciences.org/articles/00666/elife-00666.pdf</resource>"
            b"</item>"
            b"</collection>"
            b"</root>"
        )
        collection.set_collection(parent, article, "crawler-based", crossref_config)
        parent_string = ElementTree.tostring(parent, "utf8")
        self.assertEqual(parent_string, expected)

    def test_crawler_based_vor(self):
        parent = Element("root")
        crossref_config = create_crossref_config("elife")
        article = Article("10.7554/eLife.00666", "Test article")
        article.manuscript = "00666"
        article.is_poa = False
        expected = (
            b"<?xml version='1.0' encoding='utf8'?>\n"
            b"<root>"
            b'<collection property="crawler-based">'
            b'<item crawler="iParadigms">'
            b"<resource>https://elifesciences.org/articles/00666</resource>"
            b"</item>"
            b"</collection>"
            b"</root>"
        )
        collection.set_collection(parent, article, "crawler-based", crossref_config)
        parent_string = ElementTree.tostring(parent, "utf8")
        self.assertEqual(parent_string, expected)


class TestDoSetCollection(unittest.TestCase):
    def setUp(self):
        self.crossref_config = create_crossref_config()

    def test_do_set_collection_no_license(self):
        """test when an article has no license"""
        # create CrossrefXML object to test
        article = Article("10.7554/eLife.00666", "Test article")
        # test assertion
        self.assertFalse(
            collection.do_set_collection(article, "text-mining", self.crossref_config)
        )

    def test_do_set_collection_empty_license(self):
        """test when an article has no license"""
        article = Article("10.7554/eLife.00666", "Test article")
        # create an empty license
        license_object = License()
        article.license = license_object
        # test assertion
        self.assertFalse(
            collection.do_set_collection(article, "text-mining", self.crossref_config)
        )

    def test_do_set_collection_with_license(self):
        """test when an article has no license"""
        article = Article("10.7554/eLife.00666", "Test article")
        # create a license with a href value
        license_object = License()
        license_object.href = "http://example.org/license.txt"
        article.license = license_object
        # test assertion
        self.assertTrue(
            collection.do_set_collection(article, "text-mining", self.crossref_config)
        )

    def test_iparadigms_preprint(self):
        "test a preprint article iparadigms setting"
        crossref_config = create_crossref_config("elife_preprint")
        article = Article("10.7554/eLife.00666", "Test article")
        article.article_type = "preprint"
        # test assertion
        self.assertTrue(
            collection.do_set_collection(article, "crawler-based", crossref_config)
        )

    def test_iparadigms_poa(self):
        "test for a poa article iparadigms setting"
        article = Article("10.7554/eLife.00666", "Test article")
        article.is_poa = True
        # test assertion
        self.assertTrue(
            collection.do_set_collection(article, "crawler-based", self.crossref_config)
        )

    def test_iparadigms_vor(self):
        "test for a poa article iparadigms setting"
        article = Article("10.7554/eLife.00666", "Test article")
        article.is_poa = False
        # test assertion
        self.assertTrue(
            collection.do_set_collection(article, "crawler-based", self.crossref_config)
        )


class TestTextMiningXml(unittest.TestCase):
    def test_do_set_collection_text_mining_xml(self):
        "test for XML text-mining setting"
        crossref_config = create_crossref_config()
        # test assertion
        self.assertTrue(collection.do_set_collection_text_mining_xml(crossref_config))

    def test_no_setting(self):
        "test if setting is blank"
        crossref_config = create_crossref_config("DEFAULT")
        # test assertion
        self.assertEqual(
            collection.do_set_collection_text_mining_xml(crossref_config), False
        )


class TestTextMiningPdf(unittest.TestCase):
    def test_do_set_collection_text_mining_pdf(self):
        "test for PDF text-mining setting"
        crossref_config = create_crossref_config()
        article = Article("10.7554/eLife.00666", "Test article")
        # self_uri
        uri = Uri()
        uri.xlink_href = "elife-06666.pdf"
        uri.content_type = "pdf"
        article.self_uri_list = [uri]
        # test assertion
        self.assertTrue(
            collection.do_set_collection_text_mining_pdf(article, crossref_config)
        )

    def test_no_iparadigms(self):
        crossref_config = create_crossref_config("DEFAULT")
        "test if setting is blank"
        article = Article("10.7554/eLife.00666", "Test article")
        # test assertion
        self.assertEqual(
            collection.do_set_collection_text_mining_pdf(article, crossref_config),
            False,
        )


class TestIParadigmsSettingName(unittest.TestCase):
    def test_iparadigms_setting_name_preprint(self):
        article = Article("10.7554/eLife.00666", "Test article")
        article.article_type = "preprint"
        self.assertEqual(
            collection.iparadigms_setting_name(article), "iparadigms_preprint_pattern"
        )

    def test_iparadigms_setting_name_poa(self):
        article = Article("10.7554/eLife.00666", "Test article")
        article.is_poa = True
        self.assertEqual(
            collection.iparadigms_setting_name(article), "iparadigms_poa_pattern"
        )

    def test_iparadigms_setting_name_vor(self):
        article = Article("10.7554/eLife.00666", "Test article")
        article.is_poa = False
        self.assertEqual(
            collection.iparadigms_setting_name(article), "iparadigms_vor_pattern"
        )

    def test_iparadigms_setting_name_unknown(self):
        article = Article("10.7554/eLife.00666", "Test article")
        article.is_poa = None
        self.assertEqual(collection.iparadigms_setting_name(article), None)


class TestIParadigms(unittest.TestCase):
    def test_do_set_collection_iparadigms(self):
        "test for iparadigms setting"
        crossref_config = create_crossref_config()
        article = Article("10.7554/eLife.00666", "Test article")
        article.is_poa = False
        # test assertion
        self.assertTrue(
            collection.do_set_collection_iparadigms(article, crossref_config)
        )

    def test_no_iparadigms(self):
        "test settings with no iparadigms"
        crossref_config = create_crossref_config("DEFAULT")
        article = Article("10.7554/eLife.00666", "Test article")
        # test assertion
        self.assertEqual(
            collection.do_set_collection_iparadigms(article, crossref_config), False
        )



import unittest
from xml.etree import ElementTree
from xml.etree.ElementTree import Element
from elifecrossref import tags


class TestAddCleanTag(unittest.TestCase):
    def test_add_clean_tag(self):
        """test add_clean_tag to check edge cases"""
        root = Element("root")
        # add the element
        tags.add_clean_tag(root, "p", "<test>")
        self.assertEqual(
            ElementTree.tostring(root).decode("utf-8"),
            "<root><p>&lt;test&gt;</p></root>",
        )
        # strange example based on eLife 28716 v2 Figure 2 caption
        root = Element("root")
        tags.add_clean_tag(
            root,
            "subtitle",
            "...polarization, <<italic>p</italic>>, and its variance...",
        )
        self.assertEqual(
            ElementTree.tostring(root).decode("utf-8"),
            "<root><subtitle>...polarization, &lt;p&gt;, and its variance...</subtitle></root>",
        )


if __name__ == "__main__":
    unittest.main()



from xml.etree.ElementTree import SubElement
from elifecrossref import collection, resource_url


def set_article_doi_data(parent, poa_article, crossref_config):
    doi_data_tag = set_doi_data(
        parent, poa_article, poa_article, crossref_config, "doi_pattern"
    )
    collection.set_collection(doi_data_tag, poa_article, "text-mining", crossref_config)
    collection.set_collection(
        doi_data_tag, poa_article, "crawler-based", crossref_config
    )


def set_doi_data(parent, obj, poa_article, crossref_config, pattern_type=None):
    doi_data_tag = SubElement(parent, "doi_data")

    doi_tag = SubElement(doi_data_tag, "doi")
    doi_tag.text = obj.doi

    resource_tag = SubElement(doi_data_tag, "resource")

    resource = resource_url.generate_resource_url(
        obj, poa_article, crossref_config, pattern_type
    )
    resource_tag.text = resource

    return doi_data_tag



import unittest
from elifearticle.article import Article
from elifecrossref import generate
from elifecrossref.conf import raw_config, parse_raw_config


class TestGenerateAbstract(unittest.TestCase):
    def setUp(self):
        self.abstract = (
            '<p><bold><italic><underline><sub><sup>An abstract. <ext-link ext-link-type="uri" '
            'xlink:href="http://dx.doi.org/10.1601/nm.3602">Desulfocapsa sulfexigens</ext-link>.'
            "</sup></sub></underline></italic></bold>"
            ' <xref ref-type="bibr" rid="bib18">Stock and Wise (1990)</xref>.</p>'
        )
        self.abstract_xml = (
            '<abstract id="foo" xlmns:mml="http://www.w3.org/1998/Math/MathML">'
            '<object-id pub-id-type="doi">10.7554/eLife.39122.001</object-id>'
            '<p><bold><italic><underline><sub><sup>An abstract. <ext-link ext-link-type="uri" '
            'xlink:href="http://dx.doi.org/10.1601/nm.3602">Desulfocapsa sulfexigens</ext-link>.'
            "</sup></sub></underline></italic></bold>"
            ' <xref ref-type="bibr" rid="bib18">Stock and Wise (1990)</xref>.</p></abstract>'
        )

    def test_set_abstract(self):
        doi = "10.7554/eLife.00666"
        title = "Test article"
        article = Article(doi, title)
        article.abstract = self.abstract
        article.abstract_xml = self.abstract_xml
        expected_contains = (
            "<jats:abstract><jats:p>An abstract. Desulfocapsa sulfexigens."
            " Stock and Wise (1990).</jats:p></jats:abstract>"
        )
        # generate
        crossref_object = generate.build_crossref_xml([article])
        crossref_xml_string = crossref_object.output_xml()
        # test assertion
        self.assertTrue(expected_contains in crossref_xml_string)

    def test_set_abstract_jats_abstract_format(self):
        """test the abstract using jats abstract format set to true"""
        doi = "10.7554/eLife.00666"
        title = "Test article"
        article = Article(doi, title)
        article.abstract = self.abstract
        article.abstract_xml = self.abstract_xml
        expected_contains = (
            '<jats:abstract xmlns:xlink="http://www.w3.org/1999/xlink">'
            '<jats:p xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<jats:bold><jats:italic><jats:underline><jats:sub><jats:sup>"
            "An abstract. "
            '<jats:ext-link ext-link-type="uri" xlink:href="http://dx.doi.org/10.1601/nm.3602">'
            "Desulfocapsa sulfexigens</jats:ext-link>."
            "</jats:sup></jats:sub></jats:underline></jats:italic></jats:bold> "
            '<jats:xref ref-type="bibr">Stock and Wise (1990)</jats:xref>.'
            "</jats:p></jats:abstract>"
        )
        # generate
        raw_config_object = raw_config("elife")
        jats_abstract = raw_config_object.get("jats_abstract")
        raw_config_object["jats_abstract"] = "true"
        crossref_config = parse_raw_config(raw_config_object)
        crossref_object = generate.CrossrefXML([article], crossref_config, None, True)
        crossref_xml_string = crossref_object.output_xml()
        # test assertion
        self.assertTrue(expected_contains in crossref_xml_string)
        # now set the config back to normal
        raw_config_object["jats_abstract"] = jats_abstract



import unittest
from xml.etree import ElementTree
from xml.etree.ElementTree import Element
from elifearticle.article import Preprint
from elifecrossref import preprint, related


class TestSetPreprint(unittest.TestCase):
    def test_set_preprint_doi(self):
        doi = "https://doi.org/10.7554/eLife.00666"
        preprint_object = Preprint(doi=doi, uri="https://ignored-value.example.org/")
        parent = Element("root")
        related_tag = related.set_relations_program(parent, None)
        expected_parent = (
            b"<?xml version='1.0' encoding='utf8'?>"
            b"\n<root>"
            b"<rel:program>"
            b"<rel:related_item>"
            b'<rel:intra_work_relation identifier-type="doi" relationship-type="hasPreprint">'
            b"%s</rel:intra_work_relation>"
            b"</rel:related_item>"
            b"</rel:program>"
            b"</root>" % bytes(doi, "utf8")
        )
        preprint.set_preprint(related_tag, preprint_object)
        self.assertEqual(ElementTree.tostring(parent, "utf8"), expected_parent)

    def test_set_preprint_uri(self):
        uri = "https://example.org"
        preprint_object = Preprint(uri=uri)
        parent = Element("root")
        related_tag = related.set_relations_program(parent, None)
        expected_parent = (
            b"<?xml version='1.0' encoding='utf8'?>"
            b"\n<root>"
            b"<rel:program>"
            b"<rel:related_item>"
            b'<rel:intra_work_relation identifier-type="uri" relationship-type="hasPreprint">'
            b"%s</rel:intra_work_relation>"
            b"</rel:related_item>"
            b"</rel:program>"
            b"</root>" % bytes(uri, "utf8")
        )
        preprint.set_preprint(related_tag, preprint_object)
        self.assertEqual(ElementTree.tostring(parent, "utf8"), expected_parent)



from xml.etree.ElementTree import SubElement


def do_citation_related_item(ref):
    """decide whether to create a related_item for a citation"""
    if ref.publication_type and ref.publication_type == "data":
        return bool(ref.doi or ref.accession or ref.pmid or ref.uri)
    return False


def do_dataset_related_item(dataset):
    """decide whether to create a related_item for a dataset"""
    return bool(dataset.accession_id or dataset.doi or dataset.uri)


def do_preprint_related_item(poa_article):
    """decide whether to create a related_item for a preprint"""
    return bool(
        hasattr(poa_article, "preprint")
        and poa_article.preprint
        and (poa_article.preprint.uri or poa_article.preprint.doi)
    )


def do_relations_program(poa_article):
    """call at a specific moment during generation to set this tag if required"""
    do_relations = None
    for dataset in poa_article.datasets:
        if do_dataset_related_item(dataset) is True:
            do_relations = True
            break
    if do_relations is not True and poa_article.ref_list:
        for ref in poa_article.ref_list:
            if do_citation_related_item(ref) is True:
                do_relations = True
                break
    if do_relations is not True and poa_article.funding_awards:
        for funding_award in poa_article.funding_awards:
            if (
                funding_award.awards
                and len(
                    [
                        award
                        for award in funding_award.awards
                        if award.award_id_type == "doi"
                    ]
                )
                > 0
            ):
                do_relations = True
                break
    if do_relations is not True and do_preprint_related_item(poa_article):
        do_relations = True
    return do_relations


def set_relations_program(parent, relations_program_tag):
    """set the relations program parent tag only once"""
    if relations_program_tag is None:
        relations_program_tag = SubElement(parent, "rel:program")
    return relations_program_tag


def set_related_item_description(parent, description):
    if description:
        description_tag = SubElement(parent, "rel:description")
        description_tag.text = description


def set_related_item_work_relation(
    parent, related_item_type, relationship_type, identifier_type, related_item_text
):
    # only supporting inter_work_relation for now
    if related_item_type in ["intra_work_relation", "inter_work_relation"]:
        work_relation_tag = SubElement(parent, "rel:%s" % related_item_type)
        work_relation_tag.set("identifier-type", identifier_type)
        work_relation_tag.set("relationship-type", relationship_type)
        work_relation_tag.text = related_item_text



import unittest
import os
import sys
from xml.etree.ElementTree import Comment
from elifecrossref import generate
from tests import (
    DEFAULT_PUB_DATE,
    TEST_BASE_PATH,
    TEST_DATA_PATH,
    read_file_content,
    create_crossref_config,
    replace_namespaces,
)


generate.TMP_DIR = TEST_BASE_PATH + "tmp" + os.sep


def contributor_orcid_authenticated(article, orcid_authenticated):
    "set the orcid_authenticated attribute of contributor objects in the article"
    for contributor in article.contributors:
        if hasattr(contributor, "orcid_authenticated"):
            contributor.orcid_authenticated = orcid_authenticated
    return article


class TestGenerate(unittest.TestCase):
    def setUp(self):
        self.passes = []
        self.passes.append(
            (
                "elife-02935-v2.xml",
                "elife-crossref-02935-20170717071707.xml",
                "elife",
                DEFAULT_PUB_DATE,
            )
        )
        self.passes.append(
            (
                "elife_poa_e02725.xml",
                "elife-crossref-02725-20170717071707.xml",
                "elife",
                DEFAULT_PUB_DATE,
            )
        )
        self.passes.append(
            (
                "elife-15743-v1.xml",
                "elife-crossref-15743-20170717071707.xml",
                "elife",
                DEFAULT_PUB_DATE,
            )
        )
        self.passes.append(
            (
                "elife-02020-v1.xml",
                "elife-crossref-02020-20170717071707.xml",
                "elife",
                DEFAULT_PUB_DATE,
            )
        )
        self.passes.append(
            (
                "elife-08206-v3.xml",
                "elife-crossref-08206-20170717071707.xml",
                "elife",
                DEFAULT_PUB_DATE,
            )
        )
        self.passes.append(
            (
                "elife-04637-v2.xml",
                "elife-crossref-04637-20170717071707.xml",
                "elife",
                DEFAULT_PUB_DATE,
            )
        )
        self.passes.append(
            (
                "elife-02043-v2.xml",
                "elife-crossref-02043-20170717071707.xml",
                "elife",
                DEFAULT_PUB_DATE,
            )
        )
        self.passes.append(
            (
                "elife-16988-v1.xml",
                "elife-crossref-16988-20170717071707.xml",
                "elife",
                DEFAULT_PUB_DATE,
            )
        )
        self.passes.append(
            (
                "elife-12444-v2.xml",
                "elife-crossref-12444-20170717071707.xml",
                "elife",
                DEFAULT_PUB_DATE,
            )
        )
        self.passes.append(
            (
                "elife-00666.xml",
                "elife-crossref-00666-20170717071707.xml",
                "elife",
                DEFAULT_PUB_DATE,
            )
        )
        self.passes.append(
            (
                "elife-11134-v2.xml",
                "elife-crossref-11134-20170717071707.xml",
                "elife",
                DEFAULT_PUB_DATE,
            )
        )
        self.passes.append(
            (
                "elife-00508-v1.xml",
                "elife-crossref-00508-20170717071707.xml",
                "elife",
                DEFAULT_PUB_DATE,
            )
        )
        self.passes.append(
            (
                "cstp77-jats.xml",
                "cstp-crossref-77-20170717071707.xml",
                "cstp",
                DEFAULT_PUB_DATE,
            )
        )
        self.passes.append(
            (
                "bmjopen-4-e003269.xml",
                "crossref-bmjopen-2013-003269-20170717071707.xml",
                "bmjopen",
                DEFAULT_PUB_DATE,
            )
        )
        self.passes.append(
            (
                "up-sta-example.xml",
                "crossref-606-20170717071707.xml",
                None,
                DEFAULT_PUB_DATE,
            )
        )

    def test_generate(self):
        for (
            article_xml_file,
            crossref_xml_file,
            config_section,
            pub_date,
        ) in self.passes:
            file_path = TEST_DATA_PATH + article_xml_file
            articles = generate.build_articles_for_crossref([file_path])
            crossref_config = None
            if config_section:
                crossref_config = create_crossref_config(config_section)
                # set orcid_authenticated values for elife articles for this test
                if config_section == "elife":
                    for i, article in enumerate(articles):
                        articles[i] = contributor_orcid_authenticated(article, True)
            # generate pretty XML
            crossref_xml = generate.crossref_xml(
                articles, crossref_config, pub_date, False, pretty=True, indent="\t"
            )
            model_crossref_xml = read_file_content(TEST_DATA_PATH + crossref_xml_file)
            if sys.version_info < (3, 8):
                # pre Python 3.8 tag attributes are in a different order
                model_crossref_xml = replace_namespaces(model_crossref_xml)
            self.assertEqual(
                crossref_xml,
                model_crossref_xml.decode("utf-8"),
                "Failed parse test on file %s" % article_xml_file,
            )

    def test_parse_do_no_pass_pub_date(self):
        """
        For test coverage build a crossrefXML object without passing in a pub_date
        """
        article_xml_file = "elife_poa_e02725.xml"
        file_path = TEST_DATA_PATH + article_xml_file
        articles = generate.build_articles_for_crossref([file_path])
        crossref_config = create_crossref_config("elife")
        crossref_object = generate.CrossrefXML(articles, crossref_config, None, True)
        self.assertIsNotNone(crossref_object.pub_date)
        self.assertIsNotNone(crossref_object.generated)
        self.assertIsNotNone(
            [tag for tag in crossref_object.root.iter() if tag is Comment]
        )
        self.assertIsNotNone(crossref_object.output_xml())

    def test_generate_jats_abstract_face_markup(self):
        """
        For coverage test JATS and inline output by overriding the config
        """
        article_xml_file = "elife-16988-v1.xml"
        file_path = TEST_DATA_PATH + article_xml_file
        articles = generate.build_articles_for_crossref([file_path])
        crossref_config = create_crossref_config("elife")
        # override config values
        crossref_config["jats_abstract"] = True
        crossref_config["face_markup"] = True
        # create the Crossref XML
        crossref_object = generate.CrossrefXML(articles, crossref_config, None, True)
        # Check for some tags we expect to find in the output
        self.assertTrue("<jats:italic>" in crossref_object.output_xml())
        self.assertTrue("</b>" in crossref_object.output_xml())

    def test_crossref_xml_to_disk(self):
        """test writing to disk for test coverage and also not pass crossref_config"""
        article_xml_file = "up-sta-example.xml"
        crossref_xml_file = "crossref-606-20170717071707.xml"
        crossref_config = None
        pub_date = DEFAULT_PUB_DATE
        file_path = TEST_DATA_PATH + article_xml_file
        # build the article object
        articles = generate.build_articles_for_crossref([file_path])
        # generate and write to disk
        generate.crossref_xml_to_disk(
            articles,
            crossref_config,
            pub_date,
            False,
            "journal",
            pretty=True,
            indent="\t",
        )
        # check the output matches
        expected_output = read_file_content(TEST_DATA_PATH + crossref_xml_file)
        if sys.version_info < (3, 8):
            # pre Python 3.8 tag attributes are in a different order
            expected_output = replace_namespaces(expected_output)
        generated_output = read_file_content(generate.TMP_DIR + crossref_xml_file)
        self.assertEqual(generated_output, expected_output)



import re
from xml.dom import minidom
from elifearticle import utils as eautils
from elifetools import utils_html
from elifetools import xmlio
from elifetools import utils as etoolsutils
from elifetools.parseJATS import XML_NAMESPACES
from elifecrossref import tags, utils


def set_abstract(parent, poa_article, crossref_config):
    if hasattr(poa_article, "abstract_xml") and poa_article.abstract_xml:
        set_abstract_tag(
            parent,
            abstract=poa_article.abstract_xml,
            abstract_type="abstract",
            jats_abstract=crossref_config.get("jats_abstract"),
        )
    elif poa_article.abstract:
        set_abstract_tag(
            parent,
            abstract=poa_article.abstract,
            abstract_type="abstract",
            jats_abstract=crossref_config.get("jats_abstract"),
        )


def set_digest(parent, poa_article, crossref_config):
    if hasattr(poa_article, "digest") and poa_article.digest:
        set_abstract_tag(
            parent,
            abstract=poa_article.digest,
            abstract_type="executive-summary",
            jats_abstract=crossref_config.get("jats_abstract"),
        )


def get_abstract_attributes(abstract_type):
    if abstract_type == "executive-summary":
        return ["abstract-type"]
    return []


def get_abstract_attributes_text(abstract_type):
    if abstract_type == "executive-summary":
        return ' abstract-type="executive-summary" '
    return ""


def replace_jats_tag(from_tag_name, to_tag_name, string):
    pattern_from = r"(<%s)(.*?)>" % from_tag_name
    pattern_to = r"<%s\g<2>>" % to_tag_name
    string = re.sub(pattern_from, pattern_to, string)
    string = eautils.replace_tags(string, from_tag_name, to_tag_name)
    return string


def remove_tag_attr(attr_name, string):
    pattern_from = r'\s+%s=".*?"' % attr_name
    return re.sub(pattern_from, "", string)


def convert_sec_tags(string):
    # convert section title tags to paragraphs
    string = replace_jats_tag("title", "jats:p", string)
    return etoolsutils.remove_tag("sec", string)


def get_basic_abstract(abstract):
    # Strip inline tags, keep the p tags
    abstract = etoolsutils.remove_tag_and_text("object-id", abstract)
    abstract = etoolsutils.remove_tag("related-object", abstract)
    abstract = etoolsutils.remove_tag("abstract", abstract)
    abstract = utils_html.remove_comment_tags(abstract)
    abstract = etoolsutils.escape_ampersand(abstract)
    abstract = etoolsutils.escape_unmatched_angle_brackets(
        abstract, utils.allowed_tags()
    )
    abstract = convert_sec_tags(abstract)
    abstract = tags.clean_tags(
        abstract, do_not_clean=["<p>", "</p>", "<mml:", "</mml:"]
    )
    abstract = eautils.replace_tags(abstract, "p", "jats:p")
    return abstract


def get_jats_abstract(abstract):
    # Convert the abstract to jats abstract tags
    abstract = etoolsutils.remove_tag_and_text("object-id", abstract)
    abstract = etoolsutils.remove_tag("abstract", abstract)
    abstract = utils_html.remove_comment_tags(abstract)
    abstract = etoolsutils.escape_ampersand(abstract)
    abstract = etoolsutils.escape_unmatched_angle_brackets(
        abstract, utils.allowed_tags()
    )

    abstract = replace_jats_tag("sec", "jats:sec", abstract)
    abstract = replace_jats_tag("related-object", "jats:related-object", abstract)
    abstract = replace_jats_tag("title", "jats:title", abstract)

    abstract = eautils.replace_tags(abstract, "p", "jats:p")
    abstract = eautils.replace_tags(abstract, "italic", "jats:italic")
    abstract = eautils.replace_tags(abstract, "bold", "jats:bold")
    abstract = eautils.replace_tags(abstract, "underline", "jats:underline")
    abstract = eautils.replace_tags(abstract, "sub", "jats:sub")
    abstract = eautils.replace_tags(abstract, "sup", "jats:sup")
    abstract = eautils.replace_tags(abstract, "sc", "jats:sc")

    abstract = replace_jats_tag("inline-formula", "jats:inline-formula", abstract)
    abstract = replace_jats_tag("ext-link", "jats:ext-link", abstract)
    abstract = replace_jats_tag("xref", "jats:xref", abstract)

    # remove rid attributes
    abstract = remove_tag_attr("rid", abstract)

    return abstract


def set_abstract_tag(parent, abstract, abstract_type=None, jats_abstract=False):

    tag_name = "jats:abstract"

    attributes = get_abstract_attributes(abstract_type)
    attributes_text = get_abstract_attributes_text(abstract_type)

    if jats_abstract is True:
        tag_converted_abstract = get_jats_abstract(abstract)
    else:
        tag_converted_abstract = get_basic_abstract(abstract)

    tag_converted_abstract = re.sub(">\n", ">", tag_converted_abstract)

    minidom_tag = xmlio.reparsed_tag(
        tag_name, tag_converted_abstract, attributes_text=attributes_text
    )

    # add extra namespace attributes to jats:p tags if applicable
    for p_tag in minidom_tag.getElementsByTagName("jats:p"):
        if p_tag.hasChildNodes():
            attributes_added = add_namespace_attributes(p_tag)
            for attribute in attributes_added:
                if attribute not in attributes:
                    attributes.append(attribute)

    tags.append_tag(parent, minidom_tag, attributes=attributes)


def add_namespace_attributes(minidom_element):
    "add namespace attributes to the minidom Element if it contains namespaced tags or attributes"
    attributes = []
    tag_names, attribute_names = child_element_value_names(minidom_element)
    for namespace in XML_NAMESPACES:
        for tag_name in set.union(tag_names, attribute_names):
            if tag_name.startswith(namespace.get("prefix")):
                minidom_element.setAttributeNS(
                    namespace.get("uri"),
                    namespace.get("attribute"),
                    namespace.get("uri"),
                )
                attributes.append(namespace.get("attribute"))
    return attributes


def child_element_value_names(minidom_tag, tag_names=None, attribute_names=None):
    "recursively get a list of all child element tag names and attribute names"

    if tag_names is None:
        tag_names = set()

    if attribute_names is None:
        attribute_names = set()

    # process the Element nodes only
    for child_element in [
        child for child in minidom_tag.childNodes if isinstance(child, minidom.Element)
    ]:
        tag_names.add(child_element.tagName)
        for i in range(0, child_element.attributes.length):
            attribute_names.add(child_element.attributes.item(i).name)

        if child_element.hasChildNodes():
            # call again recursively for all other child nodes
            tag_names, attribute_names = child_element_value_names(
                child_element, tag_names, attribute_names
            )

    return tag_names, attribute_names



from xml.etree.ElementTree import SubElement
from elifecrossref import mime_type, resource_url, tags


def set_component_list(parent, poa_article, crossref_config):
    """
    Set the component_list from the article object component_list objects
    """
    if not poa_article.component_list:
        return

    component_list_tag = SubElement(parent, "component_list")
    # ignore excluded components based on the configuration settings
    component_list = [
        comp
        for comp in poa_article.component_list
        if comp.type not in crossref_config.get("component_exclude_types", [])
    ]
    for comp in component_list:
        set_component(component_list_tag, poa_article, comp, crossref_config)


def set_component(parent, poa_article, comp, crossref_config):
    component_tag = SubElement(parent, "component")
    component_tag.set("parent_relation", "isPartOf")

    set_component_titles(component_tag, comp, crossref_config)
    set_component_mime_type(component_tag, comp)
    set_component_permissions(component_tag, comp, crossref_config)
    set_component_doi(component_tag, poa_article, comp, crossref_config)


def set_component_titles(parent, comp, crossref_config):
    titles_tag = SubElement(parent, "titles")
    title_tag = SubElement(titles_tag, "title")
    title_tag.text = comp.title
    if comp.subtitle:
        set_subtitle(titles_tag, comp, crossref_config.get("face_markup"))


def set_component_mime_type(parent, comp):
    if comp.mime_type:
        # Convert to allowed mime types for Crossref, if found
        if mime_type.crossref_mime_type(comp.mime_type):
            format_tag = SubElement(parent, "format")
            format_tag.set("mime_type", mime_type.crossref_mime_type(comp.mime_type))


def set_component_permissions(parent, comp, crossref_config):
    """Specific license for the component"""
    license_href = crossref_config.get("component_license_ref")
    # First check if a license should be added
    if not license_href or not comp.permissions:
        return
    if do_set_component_permissions(comp):
        component_ai_program_tag = SubElement(parent, "ai:program")
        component_ai_program_tag.set("name", "AccessIndicators")
        license_ref_tag = SubElement(component_ai_program_tag, "ai:license_ref")
        license_ref_tag.text = license_href


def do_set_component_permissions(comp):
    """decide whether to set a component permissions"""
    if not comp.permissions:
        return None
    for permission in comp.permissions:
        # set the component permissions if it has any copyright statement or license value
        if permission.get("copyright_statement") or permission.get("license"):
            return True
    return False


def set_component_doi(parent, poa_article, comp, crossref_config):
    if not comp.doi:
        return
    # Try generating a resource value then continue
    resource = resource_url.generate_resource_url(
        comp, poa_article, crossref_config, "component_doi_pattern"
    )
    if resource:
        doi_data_tag = SubElement(parent, "doi_data")
        doi_tag_tag = SubElement(doi_data_tag, "doi")
        doi_tag_tag.text = comp.doi
        resource_tag = SubElement(doi_data_tag, "resource")
        resource_tag.text = resource


def set_subtitle(parent, component, face_markup=None):
    tag_name = "subtitle"
    # Use <i> tags, not <italic> tags, <b> tags not <bold>
    if component.subtitle:
        if face_markup is True:
            tags.add_inline_tag(parent, tag_name, component.subtitle)
        else:
            tags.add_clean_tag(parent, tag_name, component.subtitle)



import unittest
from elifecrossref import dataset


class TestDataset(unittest.TestCase):
    def test_choose_dataset_identifier_none(self):
        """test when an object has no attributes"""
        self.assertIsNone(dataset.choose_dataset_identifier(None))


if __name__ == "__main__":
    unittest.main()



from setuptools import setup

import elifecrossref

with open("README.rst") as fp:
    README = fp.read()

setup(
    name="elifecrossref",
    version=elifecrossref.__version__,
    description="eLife Crossref deposit of journal articles.",
    long_description=README,
    long_description_content_type="text/x-rst",
    packages=["elifecrossref"],
    license="MIT",
    install_requires=[
        "elifetools>=0.33.0",
        "elifearticle>=0.16.0",
        "GitPython",
        "configparser",
        "requests",
    ],
    url="https://github.com/elifesciences/elife-crossref-xml-generation",
    maintainer="eLife Sciences Publications Ltd.",
    maintainer_email="py@elifesciences.org",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
    ],
)



import unittest
from xml.etree import ElementTree
from xml.etree.ElementTree import Element
from elifearticle.article import Affiliation, Contributor
from elifecrossref import contributor


class TestSetPersonName(unittest.TestCase):
    def setUp(self):
        pass

    def test_set_person_name(self):
        """all person name fields"""
        contributor_object = Contributor(None, "Aardvark", "Adam")
        contributor_object.suffix = "Jr."
        contributor_role = "author"
        sequence = "first"
        parent_tag = Element("person_name")
        expected = (
            '<person_name contributor_role="author" sequence="first">'
            "<given_name>Adam</given_name>"
            "<surname>Aardvark</surname><suffix>Jr.</suffix></person_name>"
        )
        person_name_element = contributor.set_person_name(
            parent_tag, contributor_object, contributor_role, sequence
        )
        rough_string = ElementTree.tostring(person_name_element).decode("utf-8")
        self.assertEqual(rough_string, expected)

    def test_set_person_name_no_given_name(self):
        """all person name fields"""
        contributor_object = Contributor(None, "Aardvark", "")
        contributor_role = "author"
        sequence = "first"
        parent_tag = Element("person_name")
        expected = (
            '<person_name contributor_role="author" sequence="first">'
            "<surname>Aardvark</surname></person_name>"
        )
        person_name_element = contributor.set_person_name(
            parent_tag, contributor_object, contributor_role, sequence
        )
        rough_string = ElementTree.tostring(person_name_element).decode("utf-8")
        self.assertEqual(rough_string, expected)


class TestSetAffiliations(unittest.TestCase):
    def test_set_affiliations(self):
        parent_tag = Element("person_name")
        contributor_object = Contributor(None, "Aardvark", "")
        aff_object = Affiliation()
        aff_object.text = "Text"
        aff_object.department = "Department"
        aff_object.institution = "Institution"
        aff_object.city = "City"
        aff_object.country = "Country"
        aff_object.ror = "https://ror.org/example"
        contributor_object.set_affiliation(aff_object)
        expected = (
            "<person_name>"
            "<affiliations>"
            "<institution>"
            "<institution_name>Department, Institution</institution_name>"
            '<institution_id type="ror">https://ror.org/example</institution_id>'
            "<institution_place>City, Country</institution_place>"
            "</institution>"
            "</affiliations>"
            "</person_name>"
        )
        contributor.set_affiliations(parent_tag, contributor_object)
        rough_string = ElementTree.tostring(parent_tag).decode("utf-8")
        self.assertEqual(rough_string, expected)


class TestAffiliationNamePlace(unittest.TestCase):
    def test_affiliation_name_place_all(self):
        aff_object = Affiliation()
        aff_object.text = "Text"
        aff_object.department = "Department"
        aff_object.institution = "Institution"
        aff_object.city = "City"
        aff_object.country = "Country"
        expected_text_name = "Department, Institution"
        expected_text_place = "City, Country"
        text_name, text_place = contributor.affiliation_name_place(aff_object)
        self.assertEqual(text_name, expected_text_name)
        self.assertEqual(text_place, expected_text_place)

    def test_affiliation_name_place_text_only(self):
        aff_object = Affiliation()
        aff_object.text = "Text"
        expected_text_name = "Text"
        expected_text_place = ""
        text_name, text_place = contributor.affiliation_name_place(aff_object)
        self.assertEqual(text_name, expected_text_name)
        self.assertEqual(text_place, expected_text_place)

    def test_affiliation_name_place_place_only(self):
        aff_object = Affiliation()
        aff_object.city = "City"
        aff_object.country = "Country"
        expected_text_name = "City, Country"
        expected_text_place = ""
        text_name, text_place = contributor.affiliation_name_place(aff_object)
        self.assertEqual(text_name, expected_text_name)
        self.assertEqual(text_place, expected_text_place)


if __name__ == "__main__":
    unittest.main()



from elifearticle.article import Article, Component
from elifecrossref import elife


def generate_resource_url(obj, poa_article, crossref_config, pattern_type=None):
    # Generate a resource value for doi_data based on the object provided

    # by default use the self-uri value of an Article first
    if not isinstance(obj, Component) and pattern_type in ["doi_pattern", "peer_review_doi_pattern"]:
        for self_uri in obj.self_uri_list:
            # get it from the self-uri value that has no content_type
            if self_uri.content_type is None:
                return self_uri.xlink_href

    if isinstance(obj, Component) or pattern_type == "peer_review_doi_pattern":
        if (
            pattern_type == "component_doi_pattern"
            and crossref_config.get("elife_style_component_doi") is True
        ):
            id_value, prefix1 = elife.elife_style_component_attributes(obj)
        else:
            id_value = obj.id
            prefix1 = ""
        return crossref_config.get(pattern_type).format(
            doi=poa_article.doi,
            manuscript=poa_article.manuscript,
            volume=poa_article.volume,
            prefix1=prefix1,
            id=id_value,
        )
    if isinstance(obj, Article):
        version = elife.elife_style_article_attributes(obj)
        if crossref_config.get(pattern_type):
            return crossref_config.get(pattern_type).format(
                doi=obj.doi,
                manuscript=obj.manuscript,
                volume=obj.volume,
                version=version,
            )

    return ""



import unittest
import time
import sys
from elifearticle.article import (
    Article,
    ArticleDate,
    Uri,
)
from elifecrossref import generate
from tests import (
    DEFAULT_PUB_DATE,
    TEST_DATA_PATH,
    helpers,
    read_file_content,
    create_crossref_config,
    replace_namespaces,
)


class TestGeneratePostedContentMinimal(unittest.TestCase):
    def test_generate_minimal(self):
        "minimal data to generate a posted_content deposit"
        article = Article("10.7554/eLife.202200002")
        article.manuscript = "202200002"
        article.article_type = "preprint"
        # use self_uri to set the doi resource address
        self_uri = Uri()
        self_uri.xlink_href = "https://example.org/articles/202200002"
        article.self_uri_list = [self_uri]
        # title
        article.title = (
            "Timely sleep coupling: spindle-slow wave synchrony is "
            "linked to early amyloid-β burden and predicts memory decline"
        )
        # review date
        review_date = ArticleDate(
            "posted_date", time.strptime("2022-03-03 00:00:00", "%Y-%m-%d %H:%M:%S")
        )
        article.add_date(review_date)

        crossref_xml_file = (
            "elife-crossref-preprint-posted_content-202200002-20170717071707.xml"
        )
        config_section = "elife_preprint"
        pub_date = DEFAULT_PUB_DATE

        if config_section:
            crossref_config = create_crossref_config(config_section)
        articles = [article]
        crossref_xml = generate.crossref_xml(
            articles,
            crossref_config,
            pub_date,
            False,
            "posted_content",
            pretty=True,
            indent="    ",
        )
        model_crossref_xml = read_file_content(TEST_DATA_PATH + crossref_xml_file)
        if sys.version_info < (3, 8):
            # pre Python 3.8 tag attributes are in a different order
            model_crossref_xml = replace_namespaces(model_crossref_xml)
        self.assertEqual(
            crossref_xml,
            model_crossref_xml.decode("utf-8"),
            "Failed parse test on articles %s"
            % ", ".join([article.doi for article in articles]),
        )


class TestGeneratePostedContent(unittest.TestCase):
    def setUp(self):
        article = helpers.build_preprint_article()
        # add the test to the passing test list
        self.passes = []
        self.passes.append(
            (
                [article],
                "elife-crossref-preprint-posted_content-202200001-20170717071707.xml",
                "elife_preprint",
                DEFAULT_PUB_DATE,
            )
        )

    def test_generate_posted_content(self):
        for (
            articles,
            crossref_xml_file,
            config_section,
            pub_date,
        ) in self.passes:
            crossref_config = None
            if config_section:
                crossref_config = create_crossref_config(config_section)
                crossref_config["jats_abstract"] = True
            crossref_xml = generate.crossref_xml(
                articles,
                crossref_config,
                pub_date,
                False,
                "posted_content",
                pretty=True,
                indent="    ",
            )
            model_crossref_xml = read_file_content(TEST_DATA_PATH + crossref_xml_file)
            if sys.version_info < (3, 8):
                # pre Python 3.8 tag attributes are in a different order
                model_crossref_xml = replace_namespaces(model_crossref_xml)
            self.assertEqual(
                crossref_xml,
                model_crossref_xml.decode("utf-8"),
                "Failed parse test on articles %s"
                % ", ".join([article.doi for article in articles]),
            )



import unittest
from elifecrossref import conf


class TestConf(unittest.TestCase):
    def test_load_config(self):
        """test loading when no config file is specified for test coverage"""
        self.assertIsNotNone(conf.load_config(None))



import unittest
from xml.etree import ElementTree
from xml.etree.ElementTree import Element
from elifearticle.article import Citation
from elifecrossref import citation


class TestSetUnstructuredCitation(unittest.TestCase):
    def setUp(self):
        pass

    def test_set_unstructured_citation_no_face_markup(self):
        """unstructured citation example with no face markup"""
        article_title = (
            "PhD thesis: Submicroscopic <italic>Plasmodium falciparum</italic> gametocytaemia "
            + "and the contribution to malaria transmission"
        )
        expected = (
            "<citation><unstructured_citation>PhD thesis: Submicroscopic "
            + "Plasmodium falciparum gametocytaemia and the contribution to "
            + "malaria transmission.</unstructured_citation></citation>"
        )
        citation_object = Citation()
        citation_object.article_title = article_title
        citation_object.publication_type = "patent"
        parent = Element("citation")
        face_markup = False
        citation_element = citation.set_unstructured_citation(
            parent, citation_object, face_markup
        )
        rough_string = ElementTree.tostring(citation_element).decode("utf-8")
        self.assertEqual(rough_string, expected)

    def test_set_unstructured_citation_face_markup(self):
        """unstructured citation example which does include face markup"""
        article_title = (
            "PhD thesis: Submicroscopic <italic>Plasmodium falciparum</italic> gametocytaemia "
            + "and the contribution to malaria transmission"
        )
        expected = (
            "<citation><unstructured_citation>PhD thesis: Submicroscopic "
            + "<i>Plasmodium falciparum</i> gametocytaemia and the contribution to "
            + "malaria transmission.</unstructured_citation></citation>"
        )
        citation_object = Citation()
        citation_object.article_title = article_title
        citation_object.publication_type = "patent"
        parent = Element("citation")
        face_markup = True
        citation_element = citation.set_unstructured_citation(
            parent, citation_object, face_markup
        )
        rough_string = ElementTree.tostring(citation_element).decode("utf-8")
        self.assertEqual(rough_string, expected)


class TestCitationPublisher(unittest.TestCase):
    def setUp(self):
        self.publisher_loc = "Nijmegen, The Netherlands"
        self.publisher_name = "Radboud University Nijmegen Medical Centre"

    def test_citation_publisher_none(self):
        """no publisher name concatenation"""
        citation_object = Citation()
        self.assertIsNone(citation.citation_publisher(citation_object))

    def test_citation_publisher_loc_only(self):
        """no publisher name concatenation"""
        citation_object = Citation()
        citation_object.publisher_loc = self.publisher_loc
        self.assertEqual(
            citation.citation_publisher(citation_object), "Nijmegen, The Netherlands"
        )

    def test_citation_publisher_name_only(self):
        """no publisher name concatenation"""
        citation_object = Citation()
        citation_object.publisher_name = self.publisher_name
        self.assertEqual(
            citation.citation_publisher(citation_object),
            "Radboud University Nijmegen Medical Centre",
        )

    def test_citation_publisher_all(self):
        """no publisher name concatenation"""
        citation_object = Citation()
        citation_object.publisher_loc = self.publisher_loc
        citation_object.publisher_name = self.publisher_name
        self.assertEqual(
            citation.citation_publisher(citation_object),
            "Nijmegen, The Netherlands: Radboud University Nijmegen Medical Centre",
        )


if __name__ == "__main__":
    unittest.main()



import unittest
import os
import sys
from elifecrossref import generate
from tests import (
    DEFAULT_PUB_DATE,
    TEST_BASE_PATH,
    TEST_DATA_PATH,
    read_file_content,
    create_crossref_config,
    replace_namespaces,
)


generate.TMP_DIR = TEST_BASE_PATH + "tmp" + os.sep


class TestGeneratePeerReview(unittest.TestCase):
    def setUp(self):
        self.passes = []
        self.passes.append(
            (
                "elife-00666.xml",
                "elife-crossref-pending_publication-00666-20170717071707.xml",
                "elife",
                DEFAULT_PUB_DATE,
            )
        )

    def test_generate_pending_publication(self):
        for (
            article_xml_file,
            crossref_xml_file,
            config_section,
            pub_date,
        ) in self.passes:
            file_path = TEST_DATA_PATH + article_xml_file
            articles = generate.build_articles_for_crossref([file_path])
            crossref_config = None
            if config_section:
                crossref_config = create_crossref_config(config_section)
            crossref_xml = generate.crossref_xml(
                articles,
                crossref_config,
                pub_date,
                False,
                "pending_publication",
                pretty=True,
                indent="    ",
            )
            model_crossref_xml = read_file_content(TEST_DATA_PATH + crossref_xml_file)
            if sys.version_info < (3, 8):
                # pre Python 3.8 tag attributes are in a different order
                model_crossref_xml = replace_namespaces(model_crossref_xml)
            self.assertEqual(
                crossref_xml,
                model_crossref_xml.decode("utf-8"),
                "Failed parse test on file %s" % article_xml_file,
            )
